@@ -18,6 +18,7 @@ source $LIBDIR/tag_ports_during_bulk_creation
 source $LIBDIR/octavia
 source $LIBDIR/loki
 source $LIBDIR/local_ip
+source $LIBDIR/port_trusted_vif
 
 # source the OVS/OVN compilation helper methods
 source $TOP_DIR/lib/neutron_plugins/ovs_source
@@ -98,6 +99,9 @@ if [[ "$1" == "stack" ]]; then
                 fi
                 configure_l3_agent
             fi
+            if is_service_enabled q-port-trusted-vif neutron-port-trusted-vif; then
+                configure_port_trusted_ml2_extension
+            fi
             if [ $NEUTRON_CORE_PLUGIN = ml2 ]; then
                 configure_ml2_extension_drivers
             fi
@@ -117,7 +121,15 @@ if [[ "$1" == "stack" ]]; then
             if is_service_enabled br-ex-tcpdump ; then
                 # tcpdump monitor on br-ex for ARP, reverse ARP and ICMP v4 / v6 packets
                 sudo ip link set dev $PUBLIC_BRIDGE up
-                run_process br-ex-tcpdump "/usr/sbin/tcpdump -i $PUBLIC_BRIDGE arp or rarp or icmp or icmp6 -enlX" "$STACK_GROUP" root
+                TCPDUMP=$(which tcpdump)
+                if [[ ! $TCPDUMP ]]; then
+                    if [[ "$os_CODENAME" == "jammy" ]]; then
+                        TCPDUMP=/usr/bin/tcpdump
+                    else
+                        TCPDUMP=/usr/sbin/tcpdump
+                    fi
+                fi
+                run_process br-ex-tcpdump "$TCPDUMP -i $PUBLIC_BRIDGE arp or rarp or icmp or icmp6 -enlX" "$STACK_GROUP" root
             fi
 
             if is_service_enabled br-int-flows ; then
