@@ -24,7 +24,6 @@ from neutron_lib.agent import topics
 from neutron_lib import constants
 from neutron_lib import context
 from neutron_lib import exceptions
-from neutron_lib import rpc as n_rpc
 from oslo_concurrency import lockutils
 from oslo_config import cfg
 from oslo_log import helpers as log_helpers
@@ -37,6 +36,7 @@ from oslo_utils import netutils
 from oslo_utils import timeutils
 
 from neutron._i18n import _
+from neutron.agent.common import base_agent_rpc
 from neutron.agent.common import resource_processing_queue as queue
 from neutron.agent.linux import dhcp
 from neutron.agent.linux import external_process
@@ -198,9 +198,44 @@ class DhcpAgent(manager.Manager):
             eventlet.greenthread.sleep(self.conf.bulk_reload_interval)
 
     def call_driver(self, action, network, **action_kwargs):
+        sid_segment = {}
+        sid_subnets = collections.defaultdict(list)
+        if not network:
+            LOG.info('Network not present, action: %s, action_kwargs: %s',
+                     action, action_kwargs)
+            # There is nothing we can do.
+            return
+        if action == 'get_metadata_bind_interface':
+            # Special condition, this action returns a string instead of a
+            # bool.
+            return self._call_driver(action, network, **action_kwargs)
+        if 'segments' in network and network.segments:
+            # In case of multi-segments network, let's group network per
+            # segments.  We can then create DHPC process per segmentation
+            # id. All subnets on a same network that are sharing the same
+            # segmentation id will be grouped.
+            for segment in network.segments:
+                sid_segment[segment.id] = segment
+            for subnet in network.subnets:
+                sid_subnets[subnet.get('segment_id')].append(subnet)
+        if sid_subnets:
+            ret = []
+            for seg_id, subnets in sid_subnets.items():
+                network.subnets = subnets
+                ret.append(self._call_driver(
+                    action, network, segment=sid_segment.get(seg_id),
+                    **action_kwargs))
+            return all(ret)
+        else:
+            # In case subnets are not attached to segments. default behavior.
+            return self._call_driver(
+                action, network, **action_kwargs)
+
+    def _call_driver(self, action, network, segment=None, **action_kwargs):
         """Invoke an action on a DHCP driver instance."""
-        LOG.debug('Calling driver for network: %(net)s action: %(action)s',
-                  {'net': network.id, 'action': action})
+        LOG.debug('Calling driver for network: %(net)s/seg=%(seg)s '
+                  'action: %(action)s',
+                  {'net': network.id, 'action': action, 'seg': segment})
         if self.conf.bulk_reload_interval and action == 'reload_allocations':
             LOG.debug("Call deferred to bulk load")
             self._network_bulk_allocations[network.id] = True
@@ -214,7 +249,8 @@ class DhcpAgent(manager.Manager):
                                           network,
                                           self._process_monitor,
                                           self.dhcp_version,
-                                          self.plugin_rpc)
+                                          self.plugin_rpc,
+                                          segment)
             rv = getattr(driver, action)(**action_kwargs)
             if action == 'get_metadata_bind_interface':
                 return rv
@@ -303,9 +339,9 @@ class DhcpAgent(manager.Manager):
 
     def _dhcp_ready_ports_loop(self):
         """Notifies the server of any ports that had reservations setup."""
-        while True:
+        @_wait_if_syncing
+        def dhcp_ready_ports_loop():
             # this is just watching sets so we can do it really frequently
-            eventlet.sleep(0.1)
             prio_ports_to_send = set()
             ports_to_send = set()
             for port_count in range(min(len(self.dhcp_prio_ready_ports) +
@@ -321,7 +357,7 @@ class DhcpAgent(manager.Manager):
                                                         ports_to_send)
                     LOG.info("DHCP configuration for ports %s is completed",
                              prio_ports_to_send | ports_to_send)
-                    continue
+                    return
                 except oslo_messaging.MessagingTimeout:
                     LOG.error("Timeout notifying server of ports ready. "
                               "Retrying...")
@@ -332,6 +368,10 @@ class DhcpAgent(manager.Manager):
                 self.dhcp_prio_ready_ports |= prio_ports_to_send
                 self.dhcp_ready_ports |= ports_to_send
 
+        while True:
+            eventlet.sleep(0.2)
+            dhcp_ready_ports_loop()
+
     def start_ready_ports_loop(self):
         """Spawn a thread to push changed ports to server."""
         eventlet.spawn(self._dhcp_ready_ports_loop)
@@ -563,6 +603,7 @@ class DhcpAgent(manager.Manager):
         for tmp, update in self._queue.each_update_to_next_resource():
             method = getattr(self, update.action)
             method(update.resource)
+            LOG.debug('Pending events to be processed: %s', self._queue.qsize)
 
     def port_update_end(self, context, payload):
         """Handle the port.update.end notification event."""
@@ -592,7 +633,7 @@ class DhcpAgent(manager.Manager):
 
     def reload_allocations(self, port, network, prio=False):
         LOG.info("Trigger reload_allocations for port %s on network %s",
-                 port, network)
+                 port, network.id)
         if not dhcp.port_requires_dhcp_configuration(port):
             return
         driver_action = 'reload_allocations'
@@ -655,8 +696,9 @@ class DhcpAgent(manager.Manager):
             cached_ips = {i['ip_address']
                           for i in port_cached['fixed_ips']}
             if (new_ips.intersection(cached_ips) and
-                (created_port['id'] != port_cached['id'] or
-                 created_port['mac_address'] != port_cached['mac_address'])):
+                    (created_port['id'] != port_cached['id'] or
+                     created_port['mac_address'] !=
+                     port_cached['mac_address'])):
                 resync_reason = (
                     "Duplicate IP addresses found, "
                     "Port in cache: {cache_port_id}, "
@@ -685,16 +727,19 @@ class DhcpAgent(manager.Manager):
     def _port_delete(self, payload):
         port_id = payload['port_id']
         port = self.cache.get_port_by_id(port_id)
+        network = self.cache.get_network_by_id(payload['network_id'])
         self.cache.add_to_deleted_ports(port_id)
         if not port:
+            # Let's ensure that we clean namespace from stale devices
+            self.call_driver('clean_devices', network)
             return
-        network = self.cache.get_network_by_id(port.network_id)
         self.cache.remove_port(port)
         if self._is_port_on_this_agent(port):
             # the agent's port has been deleted. disable the service
             # and add the network to the resync list to create
             # (or acquire a reserved) port.
-            self.call_driver('disable', network)
+            self.call_driver('disable', network,
+                             network_id=payload['network_id'])
             self.schedule_resync("Agent port was deleted", port.network_id)
         else:
             self.call_driver('reload_allocations', network)
@@ -746,7 +791,9 @@ class DhcpAgent(manager.Manager):
                     self._metadata_routers[network.id] = (
                         router_ports[0].device_id)
 
-        if netutils.is_ipv6_enabled():
+        need_ipv6_metadata = any(subnet.ip_version == constants.IP_VERSION_6
+                                 for subnet in network.subnets)
+        if need_ipv6_metadata and netutils.is_ipv6_enabled():
             try:
                 dhcp_ifaces = [
                     self.call_driver(
@@ -788,7 +835,7 @@ class DhcpAgent(manager.Manager):
             del self._metadata_routers[network.id]
 
 
-class DhcpPluginApi(object):
+class DhcpPluginApi(base_agent_rpc.BasePluginApi):
     """Agent side of the dhcp rpc API.
 
     This class implements the client side of an rpc interface.  The server side
@@ -808,11 +855,10 @@ class DhcpPluginApi(object):
 
     def __init__(self, topic, host):
         self.host = host
-        target = oslo_messaging.Target(
-                topic=topic,
-                namespace=constants.RPC_NAMESPACE_DHCP_PLUGIN,
-                version='1.0')
-        self.client = n_rpc.get_client(target)
+        super().__init__(
+            topic=topic,
+            namespace=constants.RPC_NAMESPACE_DHCP_PLUGIN,
+            version='1.0')
 
     @property
     def context(self):
@@ -868,6 +914,11 @@ class DhcpPluginApi(object):
         if port:
             return dhcp.DictModel(port)
 
+    def get_ports(self, port_filters):
+        ports = super().get_ports(self.context, port_filters)
+        if ports:
+            return [dhcp.DictModel(port) for port in ports]
+
     def dhcp_ready_on_ports(self, port_ids):
         """Notify the server that DHCP is configured for the port."""
         cctxt = self.client.prepare(version='1.5')
