@@ -159,10 +159,11 @@ ovn_opts = [
                        "automatically set on each subnet upon creation and "
                        "on all existing subnets when Neutron starts.\n"
                        "An empty value for a DHCP option will cause that "
-                       "option to be unset globally.\n"
+                       "option to be unset globally. Multiple values should "
+                       "be separated by semi-colon.\n"
                        "EXAMPLES:\n"
-                       "- ntp_server:1.2.3.4,wpad:1.2.3.5 - Set ntp_server "
-                       "and wpad\n"
+                       "- ntp_server:1.2.3.4,wpad:1.2.3.5;1.2.3.6 - Set "
+                       "ntp_server and wpad\n"
                        "- ntp_server:,wpad:1.2.3.5 - Unset ntp_server and "
                        "set wpad\n"
                        "See the ovn-nb(5) man page for available options.")),
@@ -172,7 +173,8 @@ ovn_opts = [
                        "automatically set on each subnet upon creation and "
                        "on all existing subnets when Neutron starts.\n"
                        "An empty value for a DHCPv6 option will cause that "
-                       "option to be unset globally.\n"
+                       "option to be unset globally. Multiple values should "
+                       "be separated by semi-colon.\n"
                        "See the ovn-nb(5) man page for available options.")),
     cfg.BoolOpt('ovn_emit_need_to_frag',
                 default=False,
@@ -221,6 +223,14 @@ ovn_opts = [
                default=0,
                help=_('The number of seconds to keep MAC_Binding entries in '
                       'the OVN DB. 0 to disable aging.')),
+    cfg.BoolOpt('broadcast_arps_to_all_routers',
+                default=True,
+                help=_('If enabled (default) OVN will flood ARP requests to '
+                       'all attached ports on a network. If set to False, '
+                       'ARP requests are only sent to routers on that network '
+                       'if the target MAC address matches. ARP requests that '
+                       'do not match a router will only be forwarded to '
+                       'non-router ports. Supported by OVN >= 23.06.')),
     cfg.BoolOpt('ovn_router_indirect_snat',
                 default=False,
                 help=_('Whether to configure SNAT for all nested subnets '
@@ -388,5 +398,9 @@ def get_ovn_mac_binding_removal_limit():
     return str(cfg.CONF.ovn_nb_global.mac_binding_removal_limit)
 
 
+def is_broadcast_arps_to_all_routers_enabled():
+    return cfg.CONF.ovn.broadcast_arps_to_all_routers
+
+
 def is_ovn_router_indirect_snat_enabled():
     return cfg.CONF.ovn.ovn_router_indirect_snat
