@@ -12,19 +12,25 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 #
+import collections
 
+from neutron_lib.api.definitions import allowedaddresspairs as addr_apidef
+from neutron_lib.api.definitions import port as port_def
 from neutron_lib.api import validators
+from neutron_lib.callbacks import events
+from neutron_lib.callbacks import registry
+from neutron_lib.db import api as db_api
+from neutron_lib.db import resource_extend
+from neutron_lib.db import utils as db_utils
+from neutron_lib.exceptions import allowedaddresspairs as addr_exc
+from neutron_lib.objects import exceptions
+from neutron_lib.utils import net as net_utils
 
-from neutron.api.v2 import attributes as attr
-from neutron.db import db_base_plugin_v2
-
-from neutron.common import utils
-from neutron.extensions import allowedaddresspairs as addr_pair
-from neutron.objects import base as obj_base
 from neutron.objects.port.extensions import (allowedaddresspairs
-    as obj_addr_pair)
+                                             as obj_addr_pair)
 
 
+@resource_extend.has_resource_extenders
 class AllowedAddressPairsMixin(object):
     """Mixin class for allowed address pairs."""
 
@@ -32,16 +38,30 @@ class AllowedAddressPairsMixin(object):
                                               allowed_address_pairs):
         if not validators.is_attr_set(allowed_address_pairs):
             return []
+
+        desired_state = {
+            'context': context,
+            'network_id': port['network_id'],
+            'allowed_address_pairs': allowed_address_pairs,
+        }
+        # TODO(slaweq): use constant from neutron_lib.callbacks.resources once
+        # it will be available and released
+        registry.publish(
+            'allowed_address_pair', events.BEFORE_CREATE, self,
+            payload=events.DBEventPayload(
+                context,
+                resource_id=port['id'],
+                desired_state=desired_state))
         try:
-            with context.session.begin(subtransactions=True):
+            with db_api.CONTEXT_WRITER.using(context):
                 for address_pair in allowed_address_pairs:
                     # use port.mac_address if no mac address in address pair
                     if 'mac_address' not in address_pair:
                         address_pair['mac_address'] = port['mac_address']
                     # retain string format as passed through API
-                    mac_address = utils.AuthenticEUI(
+                    mac_address = net_utils.AuthenticEUI(
                         address_pair['mac_address'])
-                    ip_address = utils.AuthenticIPNetwork(
+                    ip_address = net_utils.AuthenticIPNetwork(
                         address_pair['ip_address'])
                     pair_obj = obj_addr_pair.AllowedAddressPair(
                         context,
@@ -49,52 +69,58 @@ class AllowedAddressPairsMixin(object):
                         mac_address=mac_address,
                         ip_address=ip_address)
                     pair_obj.create()
-        except obj_base.NeutronDbObjectDuplicateEntry:
-            raise addr_pair.DuplicateAddressPairInRequest(
+        except exceptions.NeutronDbObjectDuplicateEntry:
+            raise addr_exc.DuplicateAddressPairInRequest(
                 mac_address=address_pair['mac_address'],
                 ip_address=address_pair['ip_address'])
 
         return allowed_address_pairs
 
     def get_allowed_address_pairs(self, context, port_id):
-        pairs = self._get_allowed_address_pairs_objs(context, port_id)
+        pairs = obj_addr_pair.AllowedAddressPair.get_objects(
+            context, port_id=port_id)
         return [self._make_allowed_address_pairs_dict(pair.db_obj)
                 for pair in pairs]
 
-    def _get_allowed_address_pairs_objs(self, context, port_id):
-        pairs = obj_addr_pair.AllowedAddressPair.get_objects(
-            context, port_id=port_id)
-        return pairs
+    def get_allowed_address_pairs_for_ports(self, context, port_ids):
+        pairs = (
+            obj_addr_pair.AllowedAddressPair.
+            get_allowed_address_pairs_for_ports(
+                context, port_ids=port_ids))
+        result = collections.defaultdict(list)
+        for pair in pairs:
+            result[pair.port_id].append(
+                self._make_allowed_address_pairs_dict(pair.db_obj))
+        return result
 
-    def _extend_port_dict_allowed_address_pairs(self, port_res, port_db):
+    @staticmethod
+    @resource_extend.extends([port_def.COLLECTION_NAME])
+    def _extend_port_dict_allowed_address_pairs(port_res, port_db):
         # If port_db is provided, allowed address pairs will be accessed via
         # sqlalchemy models. As they're loaded together with ports this
         # will not cause an extra query.
         allowed_address_pairs = [
-            self._make_allowed_address_pairs_dict(address_pair) for
+            AllowedAddressPairsMixin._make_allowed_address_pairs_dict(
+                address_pair) for
             address_pair in port_db.allowed_address_pairs]
-        port_res[addr_pair.ADDRESS_PAIRS] = allowed_address_pairs
+        port_res[addr_apidef.ADDRESS_PAIRS] = allowed_address_pairs
         return port_res
 
-    # Register dict extend functions for ports
-    db_base_plugin_v2.NeutronDbPluginV2.register_dict_extend_funcs(
-        attr.PORTS, ['_extend_port_dict_allowed_address_pairs'])
-
     def _delete_allowed_address_pairs(self, context, id):
-        pairs = self._get_allowed_address_pairs_objs(context, port_id=id)
-        with context.session.begin(subtransactions=True):
-            for pair in pairs:
-                pair.delete()
+        obj_addr_pair.AllowedAddressPair.delete_objects(
+            context, port_id=id)
 
-    def _make_allowed_address_pairs_dict(self, allowed_address_pairs,
+    @staticmethod
+    def _make_allowed_address_pairs_dict(allowed_address_pairs,
                                          fields=None):
         res = {'mac_address': allowed_address_pairs['mac_address'],
                'ip_address': allowed_address_pairs['ip_address']}
-        return self._fields(res, fields)
+        return db_utils.resource_fields(res, fields)
 
     def _has_address_pairs(self, port):
-        return (validators.is_attr_set(port['port'][addr_pair.ADDRESS_PAIRS])
-                and port['port'][addr_pair.ADDRESS_PAIRS] != [])
+        return (validators.is_attr_set(
+            port['port'][addr_apidef.ADDRESS_PAIRS]) and
+                port['port'][addr_apidef.ADDRESS_PAIRS] != [])
 
     def _check_update_has_allowed_address_pairs(self, port):
         """Determine if request has an allowed address pair.
@@ -102,7 +128,7 @@ class AllowedAddressPairsMixin(object):
         Return True if the port parameter has a non-empty
         'allowed_address_pairs' attribute. Otherwise returns False.
         """
-        return (addr_pair.ADDRESS_PAIRS in port['port'] and
+        return (addr_apidef.ADDRESS_PAIRS in port['port'] and
                 self._has_address_pairs(port))
 
     def _check_update_deletes_allowed_address_pairs(self, port):
@@ -111,7 +137,7 @@ class AllowedAddressPairsMixin(object):
         Return True if port has an allowed address pair and its value
         is either [] or not is_attr_set, otherwise return False
         """
-        return (addr_pair.ADDRESS_PAIRS in port['port'] and
+        return (addr_apidef.ADDRESS_PAIRS in port['port'] and
                 not self._has_address_pairs(port))
 
     def is_address_pairs_attribute_updated(self, port, update_attrs):
@@ -122,10 +148,10 @@ class AllowedAddressPairsMixin(object):
         party controllers.
         """
 
-        new_pairs = update_attrs.get(addr_pair.ADDRESS_PAIRS)
+        new_pairs = update_attrs.get(addr_apidef.ADDRESS_PAIRS)
         if new_pairs is None:
             return False
-        old_pairs = port.get(addr_pair.ADDRESS_PAIRS)
+        old_pairs = port.get(addr_apidef.ADDRESS_PAIRS)
 
         # Missing or unchanged address pairs in attributes mean no update
         return new_pairs != old_pairs
@@ -139,11 +165,11 @@ class AllowedAddressPairsMixin(object):
         notification. This method is expected to be called within
         a transaction.
         """
-        new_pairs = port['port'].get(addr_pair.ADDRESS_PAIRS)
+        new_pairs = port['port'].get(addr_apidef.ADDRESS_PAIRS)
 
         if self.is_address_pairs_attribute_updated(original_port,
                                                    port['port']):
-            updated_port[addr_pair.ADDRESS_PAIRS] = new_pairs
+            updated_port[addr_apidef.ADDRESS_PAIRS] = new_pairs
             self._delete_allowed_address_pairs(context, port_id)
             self._process_create_allowed_address_pairs(
                 context, updated_port, new_pairs)
