@@ -23,6 +23,7 @@ import threading
 import types
 import uuid
 
+import netaddr
 from neutron_lib.api.definitions import portbindings
 from neutron_lib.api.definitions import provider_net
 from neutron_lib.api.definitions import segment as segment_def
@@ -143,6 +144,7 @@ class OVNMechanismDriver(api.MechanismDriver):
         self.qos_driver = qos_driver.OVNQosDriver.create(self)
         self.trunk_driver = trunk_driver.OVNTrunkDriver.create(self)
         self.log_driver = log_driver.register(self)
+        self._agent_cache = None
 
     @property
     def nb_schema_helper(self):
@@ -256,8 +258,13 @@ class OVNMechanismDriver(api.MechanismDriver):
         registry.subscribe(self.delete_segment_provnet_port,
                            resources.SEGMENT,
                            events.AFTER_DELETE)
+        # TODO(slaweq): use constant from neutron_lib.callbacks.resources once
+        # it will be available and released
+        registry.subscribe(self._validate_allowed_address_pairs,
+                           'allowed_address_pair',
+                           events.BEFORE_CREATE)
 
-        # Handle security group/rule notifications
+        # Handle security group/rule or address group notifications
         if self.sg_enabled:
             registry.subscribe(self._create_security_group_precommit,
                                resources.SECURITY_GROUP,
@@ -283,6 +290,15 @@ class OVNMechanismDriver(api.MechanismDriver):
             registry.subscribe(self._process_sg_rule_notification,
                                resources.SECURITY_GROUP_RULE,
                                events.BEFORE_DELETE)
+            registry.subscribe(self._process_ag_notification,
+                               resources.ADDRESS_GROUP,
+                               events.AFTER_CREATE)
+            registry.subscribe(self._process_ag_notification,
+                               resources.ADDRESS_GROUP,
+                               events.AFTER_UPDATE)
+            registry.subscribe(self._process_ag_notification,
+                               resources.ADDRESS_GROUP,
+                               events.AFTER_DELETE)
 
     def _remove_node_from_hash_ring(self, *args, **kwargs):
         # The node_uuid attribute will be empty for worker types
@@ -347,7 +363,8 @@ class OVNMechanismDriver(api.MechanismDriver):
         if worker_class == wsgi.WorkerService:
             self._setup_hash_ring()
 
-        n_agent.AgentCache(self)  # Initialize singleton agent cache
+        # Initialize singleton agent cache and keep a copy.
+        self._agent_cache = n_agent.AgentCache(self)
         self.nb_ovn, self.sb_ovn = impl_idl_ovn.get_ovn_idls(self, trigger)
 
         # Override agents API methods
@@ -502,6 +519,25 @@ class OVNMechanismDriver(api.MechanismDriver):
                 return True
         return False
 
+    def _process_ag_notification(
+            self, resource, event, trigger, payload):
+        context = payload.context
+        address_group = payload.latest_state
+        address_group_id = payload.resource_id
+        if event == events.AFTER_CREATE:
+            ovn_revision_numbers_db.create_initial_revision(
+                context, address_group_id, ovn_const.TYPE_ADDRESS_GROUPS,
+                std_attr_id=address_group['standard_attr_id'])
+            self._ovn_client.create_address_group(
+                context, address_group)
+        elif event == events.AFTER_UPDATE:
+            self._ovn_client.update_address_group(
+                context, address_group)
+        elif event == events.AFTER_DELETE:
+            self._ovn_client.delete_address_group(
+                context,
+                address_group_id)
+
     def _is_network_type_supported(self, network_type):
         return (network_type in [const.TYPE_LOCAL,
                                  const.TYPE_FLAT,
@@ -539,6 +575,57 @@ class OVNMechanismDriver(api.MechanismDriver):
                 )
                 raise n_exc.InvalidInput(error_message=m)
 
+    def _validate_allowed_address_pairs(self, resource, event, trigger,
+                                        payload):
+        context = payload.desired_state['context']
+        allowed_address_pairs = payload.desired_state['allowed_address_pairs']
+        network_id = payload.desired_state['network_id']
+        if not allowed_address_pairs:
+            return
+
+        port_allowed_address_pairs_ip_addresses = [
+            netaddr.IPNetwork(pair['ip_address'])
+            for pair in allowed_address_pairs]
+
+        distributed_ports = self._plugin.get_ports(
+            context.elevated(),
+            filters={'device_owner': [const.DEVICE_OWNER_DISTRIBUTED],
+                     'network_id': [network_id]})
+        if not distributed_ports:
+            return
+
+        def _get_common_ips(ip_addresses, ip_networks):
+            common_ips = set()
+            for ip_address in ip_addresses:
+                if any(ip_address in ip_net for ip_net in ip_networks):
+                    common_ips.add(str(ip_address))
+            return common_ips
+
+        # NOTE(slaweq): We can safely ignore any CIDR larger than /32 (for
+        # IPv4) or /128 (for IPv6) in the allowed_address_pairs, since such
+        # CIDRs cannot be set as a Virtual IP in OVN.
+        # Only /32 and /128 CIDRs are allowed to be set as Virtual IPs in OVN.
+        address_pairs_to_check = [
+            ip_net for ip_net in port_allowed_address_pairs_ip_addresses
+            if ip_net.size == 1]
+
+        for distributed_port in distributed_ports:
+            distributed_port_ip_addresses = [
+                netaddr.IPAddress(fixed_ip['ip_address']) for fixed_ip in
+                distributed_port.get('fixed_ips', [])]
+
+            common_ips = _get_common_ips(
+                distributed_port_ip_addresses,
+                address_pairs_to_check)
+
+            if common_ips:
+                err_msg = (
+                    _("IP addresses '%s' already used by the '%s' port(s) in "
+                      "the same network" % (";".join(common_ips),
+                                            const.DEVICE_OWNER_DISTRIBUTED))
+                )
+                raise n_exc.InvalidInput(error_message=err_msg)
+
     def create_segment_provnet_port(self, resource, event, trigger,
                                     payload=None):
         segment = payload.latest_state
@@ -1237,7 +1324,7 @@ class OVNMechanismDriver(api.MechanismDriver):
 
             if self._should_notify_nova(db_port):
                 self._plugin.nova_notifier.record_port_status_changed(
-                    db_port, const.PORT_STATUS_ACTIVE, const.PORT_STATUS_DOWN,
+                    db_port, const.PORT_STATUS_DOWN, const.PORT_STATUS_ACTIVE,
                     None)
                 self._plugin.nova_notifier.send_port_status(
                     None, None, db_port)
@@ -1490,11 +1577,6 @@ def delete_agent(self, context, id, _driver=None):
         'SB_Global', '.', 'external_ids', delete_agent=str(id),
         if_exists=True).execute(check_error=True)
 
-    try:
-        n_agent.AgentCache().delete(id)
-    except KeyError:
-        LOG.debug('OVN agent %s has been deleted concurrently', id)
-
 
 def get_availability_zones(cls, context, _driver, filters=None, fields=None,
                            sorts=None, limit=None, marker=None,
