@@ -15,6 +15,8 @@
 from oslo_utils import timeutils
 from ovsdbapp.backend.ovs_idl import command
 from ovsdbapp.backend.ovs_idl import idlutils
+from ovsdbapp.schema.ovn_northbound import commands as ovn_nb_commands
+from ovsdbapp import utils as ovsdbapp_utils
 
 from neutron._i18n import _
 from neutron.common.ovn import constants as ovn_const
@@ -22,6 +24,11 @@ from neutron.common.ovn import exceptions as ovn_exc
 from neutron.common.ovn import utils
 from neutron.services.portforwarding.constants import PORT_FORWARDING_PREFIX
 
+from oslo_log import log
+
+
+LOG = log.getLogger(__name__)
+
 RESOURCE_TYPE_MAP = {
     ovn_const.TYPE_NETWORKS: 'Logical_Switch',
     ovn_const.TYPE_PORTS: 'Logical_Switch_Port',
@@ -29,6 +36,7 @@ RESOURCE_TYPE_MAP = {
     ovn_const.TYPE_ROUTER_PORTS: 'Logical_Router_Port',
     ovn_const.TYPE_FLOATINGIPS: 'NAT',
     ovn_const.TYPE_SUBNETS: 'DHCP_Options',
+    ovn_const.TYPE_ADDRESS_GROUPS: 'Address_Set',
 }
 
 
@@ -81,6 +89,11 @@ def _add_gateway_chassis(api, txn, lrp_name, val):
                 gwc.name = gwc_name
             gwc.chassis_name = chassis
             gwc.priority = prio
+            LOG.info(
+                "Schedule LRP %(lrp)s on gateway %(gtw)s with priority"
+                " %(prio)s",
+                {"lrp": lrp_name, "gtw": chassis, "prio": prio},
+            )
             prio = prio - 1
             uuid_list.append(gwc.uuid)
         return 'gateway_chassis', uuid_list
@@ -122,6 +135,7 @@ class AddLSwitchPortCommand(command.BaseCommand):
                                          'Logical_Switch_Port', 'name',
                                          self.lport, None)
             if port:
+                self.result = port.uuid
                 return
 
         port = txn.insert(self.api._tables['Logical_Switch_Port'])
@@ -137,6 +151,14 @@ class AddLSwitchPortCommand(command.BaseCommand):
             port.dhcpv6_options = dhcpv6_options
         else:
             port.dhcpv6_options = [dhcpv6_options.result]
+
+        # NOTE(ralonsoh): HA chassis group is created by Neutron, there is no
+        # need to create it in this command.
+        ha_chassis_group = self.columns.pop('ha_chassis_group', None)
+        if ha_chassis_group:
+            hcg_uuid = ovsdbapp_utils.get_uuid(ha_chassis_group)
+            port.ha_chassis_group = hcg_uuid
+
         for col, val in self.columns.items():
             setattr(port, col, val)
         # add the newly created port to existing lswitch
@@ -148,9 +170,10 @@ class AddLSwitchPortCommand(command.BaseCommand):
 
 
 class SetLSwitchPortCommand(command.BaseCommand):
-    def __init__(self, api, lport, if_exists, **columns):
+    def __init__(self, api, lport, external_ids_update, if_exists, **columns):
         super(SetLSwitchPortCommand, self).__init__(api)
         self.lport = lport
+        self.external_ids_update = external_ids_update
         self.columns = columns
         self.if_exists = if_exists
 
@@ -195,6 +218,27 @@ class SetLSwitchPortCommand(command.BaseCommand):
         for uuid in cur_port_dhcp_opts - new_port_dhcp_opts:
             self.api._tables['DHCP_Options'].rows[uuid].delete()
 
+        external_ids_update = self.external_ids_update or {}
+        external_ids = getattr(port, 'external_ids', {})
+        for k, v in external_ids_update.items():
+            external_ids[k] = v
+        port.external_ids = external_ids
+
+        # NOTE(ralonsoh): HA chassis group is created by Neutron, there is no
+        # need to create it in this command. The register is also deleted when
+        # the network to which the HA chassis group is associated is deleted.
+        ha_chassis_group = self.columns.pop('ha_chassis_group', None)
+        if ha_chassis_group:
+            hcg_uuid = ovsdbapp_utils.get_uuid(ha_chassis_group)
+            try:
+                port_hcg_uuid = port.ha_chassis_group[0].uuid
+            except IndexError:
+                port_hcg_uuid = None
+            if port_hcg_uuid != hcg_uuid:
+                port.ha_chassis_group = hcg_uuid
+        elif ha_chassis_group == []:
+            port.ha_chassis_group = []
+
         for col, val in self.columns.items():
             setattr(port, col, val)
 
@@ -256,26 +300,6 @@ class DelLSwitchPortCommand(command.BaseCommand):
         self.api._tables['Logical_Switch_Port'].rows[lport.uuid].delete()
 
 
-class AddLRouterCommand(command.BaseCommand):
-    def __init__(self, api, name, may_exist, **columns):
-        super(AddLRouterCommand, self).__init__(api)
-        self.name = name
-        self.columns = columns
-        self.may_exist = may_exist
-
-    def run_idl(self, txn):
-        if self.may_exist:
-            lrouter = idlutils.row_by_value(self.api.idl, 'Logical_Router',
-                                            'name', self.name, None)
-            if lrouter:
-                return
-
-        row = txn.insert(self.api._tables['Logical_Router'])
-        row.name = self.name
-        for col, val in self.columns.items():
-            setattr(row, col, val)
-
-
 class UpdateLRouterCommand(command.BaseCommand):
     def __init__(self, api, name, if_exists, **columns):
         super(UpdateLRouterCommand, self).__init__(api)
@@ -299,23 +323,111 @@ class UpdateLRouterCommand(command.BaseCommand):
             return
 
 
-class DelLRouterCommand(command.BaseCommand):
-    def __init__(self, api, name, if_exists):
-        super(DelLRouterCommand, self).__init__(api)
-        self.name = name
-        self.if_exists = if_exists
+class ScheduleUnhostedGatewaysCommand(command.BaseCommand):
+    def __init__(self, nb_api, g_name, sb_api, plugin, port_physnets,
+                 all_gw_chassis, chassis_with_physnets, chassis_with_azs):
+        super().__init__(api=nb_api)
+        self.g_name = g_name
+        self.sb_api = sb_api
+        self.scheduler = plugin.scheduler
+        self.ovn_client = plugin._ovn_client
+        self.port_physnets = port_physnets
+        self.all_gw_chassis = all_gw_chassis
+        self.chassis_with_physnets = chassis_with_physnets
+        self.chassis_with_azs = chassis_with_azs
 
     def run_idl(self, txn):
-        try:
-            lrouter = idlutils.row_by_value(self.api.idl, 'Logical_Router',
-                                            'name', self.name)
-        except idlutils.RowNotFound:
-            if self.if_exists:
-                return
-            msg = _("Logical Router %s does not exist") % self.name
-            raise RuntimeError(msg)
+        lrouter_port = self.api.lookup("Logical_Router_Port", self.g_name)
+        physnet = self.port_physnets.get(
+            self.g_name[len(ovn_const.LRP_PREFIX):])
+        # Remove any invalid gateway chassis from the list, otherwise
+        # we can have a situation where all existing_chassis are invalid
+        existing_chassis = self.api.get_gateway_chassis_binding(self.g_name)
+        primary = existing_chassis[0] if existing_chassis else None
+        az_hints = self.api.get_gateway_chassis_az_hints(self.g_name)
+        filtered_existing_chassis = (
+            self.scheduler.filter_existing_chassis(
+                nb_idl=self.api, gw_chassis=self.all_gw_chassis,
+                physnet=physnet,
+                chassis_physnets=self.chassis_with_physnets,
+                existing_chassis=existing_chassis, az_hints=az_hints,
+                chassis_with_azs=self.chassis_with_azs))
+        if existing_chassis != filtered_existing_chassis:
+            first_diff = None
+            for i in range(len(filtered_existing_chassis)):
+                if existing_chassis[i] != filtered_existing_chassis[i]:
+                    first_diff = i
+                    break
+            if first_diff is not None:
+                LOG.debug(
+                    "A chassis for this gateway has been filtered. "
+                    "Rebalancing priorities %s and lower", first_diff)
+                filtered_existing_chassis = filtered_existing_chassis[
+                    :max(first_diff, 1)]
 
-        self.api._tables['Logical_Router'].rows[lrouter.uuid].delete()
+        candidates = self.ovn_client.get_candidates_for_scheduling(
+            physnet, cms=self.all_gw_chassis,
+            chassis_physnets=self.chassis_with_physnets,
+            availability_zone_hints=az_hints)
+        chassis = self.scheduler.select(
+            self.api, self.sb_api, self.g_name, candidates=candidates,
+            existing_chassis=filtered_existing_chassis)
+        if primary and primary != chassis[0]:
+            if primary not in chassis:
+                LOG.debug("Primary gateway chassis %(old)s "
+                          "has been removed from the system. Moving "
+                          "gateway %(gw)s to other chassis %(new)s.",
+                          {'gw': self.g_name,
+                           'old': primary,
+                           'new': chassis[0]})
+            else:
+                LOG.debug("Gateway %s is hosted at %s.", self.g_name, primary)
+                # NOTE(mjozefcz): It means scheduler moved primary chassis
+                # to other gw based on scheduling method. But we don't
+                # want network flap - so moving actual primary to be on
+                # the top.
+                index = chassis.index(primary)
+                chassis[0], chassis[index] = chassis[index], chassis[0]
+        setattr(
+            lrouter_port,
+            *_add_gateway_chassis(self.api, txn, self.g_name, chassis))
+
+
+class ScheduleNewGatewayCommand(command.BaseCommand):
+    def __init__(self, nb_api, g_name, sb_api, lrouter_name, plugin, physnet,
+                 az_hints):
+        super().__init__(nb_api)
+        self.g_name = g_name
+        self.sb_api = sb_api
+        self.lrouter_name = lrouter_name
+        self.ovn_client = plugin._ovn_client
+        self.scheduler = plugin.scheduler
+        self.physnet = physnet
+        self.az_hints = az_hints
+
+    def run_idl(self, txn):
+        lrouter = self.api.lookup("Logical_Router", self.lrouter_name)
+        lrouter_port = self.api.lookup("Logical_Router_Port", self.g_name)
+
+        candidates = self.ovn_client.get_candidates_for_scheduling(
+            self.physnet, availability_zone_hints=self.az_hints)
+        chassis = self.scheduler.select(
+            self.api, self.sb_api, self.g_name, candidates=candidates,
+            target_lrouter=lrouter)
+        if chassis:
+            setattr(lrouter_port,
+                    *_add_gateway_chassis(self.api, txn, self.g_name, chassis))
+
+
+class LrDelCommand(ovn_nb_commands.LrDelCommand):
+
+    def run_idl(self, txn):
+        super().run_idl(txn)
+        try:
+            hcg = self.api.lookup('HA_Chassis_Group', self.router)
+            hcg.delete()
+        except idlutils.RowNotFound:
+            pass
 
 
 class AddLRouterPortCommand(command.BaseCommand):
@@ -440,6 +552,40 @@ class SetLRouterPortInLSwitchPortCommand(command.BaseCommand):
         setattr(port, 'addresses', self.lsp_address)
 
 
+class SetLRouterMacAgeLimitCommand(command.BaseCommand):
+    def __init__(self, api, router, threshold):
+        super().__init__(api)
+        self.router = router
+        self.threshold = str(threshold)  # Just in case an integer sneaks in
+
+    def run_idl(self, txn):
+        # Creating a Command object that iterates over the list of Routers
+        # from inside a transaction avoids the issue of doing two
+        # transactions: one for list_rows() and the other for setting the
+        # values on routers, which would allow routers to be added and removed
+        # between the two transactions.
+        if self.router is None:
+            routers = self.api.tables["Logical_Router"].rows.values()
+        else:
+            routers = [self.api.lookup("Logical_Router", self.router)]
+
+        for router in routers:
+            # It's not technically necessary to check the value before setting
+            # it as python-ovs is smart enough to avoid sending operations to
+            # the server that would result in no change. The overhead of
+            # setkey() though is > than the overhead of checking the value here
+            try:
+                if (router.options.get(ovn_const.LR_OPTIONS_MAC_AGE_LIMIT) ==
+                        self.threshold):
+                    continue
+            except AttributeError:
+                # The Logical_Router is newly created in this txn and has no
+                # "options" set yet, which the following setkey will rectify
+                pass
+            router.setkey("options", ovn_const.LR_OPTIONS_MAC_AGE_LIMIT,
+                          self.threshold)
+
+
 class AddACLCommand(command.BaseCommand):
     def __init__(self, api, lswitch, lport, **columns):
         super(AddACLCommand, self).__init__(api)
@@ -490,9 +636,10 @@ class DelACLCommand(command.BaseCommand):
 
 
 class AddStaticRouteCommand(command.BaseCommand):
-    def __init__(self, api, lrouter, **columns):
+    def __init__(self, api, lrouter, maintain_bfd=False, **columns):
         super(AddStaticRouteCommand, self).__init__(api)
         self.lrouter = lrouter
+        self.maintain_bfd = maintain_bfd
         self.columns = columns
 
     def run_idl(self, txn):
@@ -503,18 +650,37 @@ class AddStaticRouteCommand(command.BaseCommand):
             msg = _("Logical Router %s does not exist") % self.lrouter
             raise RuntimeError(msg)
 
+        bfd_uuid = None
+        if (self.maintain_bfd and
+                'nexthop' in self.columns and
+                'output_port' in self.columns):
+            cmd = ovn_nb_commands.BFDAddCommand(self.api,
+                                                self.columns['output_port'],
+                                                self.columns['nexthop'],
+                                                may_exist=True)
+            cmd.run_idl(txn)
+            try:
+                bfd_uuid = cmd.result.uuid
+            except AttributeError:
+                # When the BFD record is created in the same transaction the
+                # post commit code that would resolve the real UUID and look up
+                # the bfd record has not run yet, and consequently the object
+                # returned by BFDAddCommand() is an UUID object.
+                bfd_uuid = cmd.result
+
         row = txn.insert(self.api._tables['Logical_Router_Static_Route'])
         for col, val in self.columns.items():
             setattr(row, col, val)
+        if bfd_uuid:
+            setattr(row, 'bfd', bfd_uuid)
         _addvalue_to_list(lrouter, 'static_routes', row.uuid)
 
 
-class DelStaticRouteCommand(command.BaseCommand):
-    def __init__(self, api, lrouter, ip_prefix, nexthop, if_exists):
-        super(DelStaticRouteCommand, self).__init__(api)
+class DelStaticRoutesCommand(command.BaseCommand):
+    def __init__(self, api, lrouter, routes, if_exists):
+        super(DelStaticRoutesCommand, self).__init__(api)
         self.lrouter = lrouter
-        self.ip_prefix = ip_prefix
-        self.nexthop = nexthop
+        self.routes = routes
         self.if_exists = if_exists
 
     def run_idl(self, txn):
@@ -527,34 +693,33 @@ class DelStaticRouteCommand(command.BaseCommand):
             msg = _("Logical Router %s does not exist") % self.lrouter
             raise RuntimeError(msg)
 
-        static_routes = getattr(lrouter, 'static_routes', [])
-        for route in static_routes:
-            ip_prefix = getattr(route, 'ip_prefix', '')
-            nexthop = getattr(route, 'nexthop', '')
-            if self.ip_prefix == ip_prefix and self.nexthop == nexthop:
-                _delvalue_from_list(lrouter, 'static_routes', route)
-                route.delete()
-                break
+        routes_to_be_deleted = []
+        for route in getattr(lrouter, 'static_routes', []):
+            route_tuple = (getattr(route, 'ip_prefix', ''),
+                           getattr(route, 'nexthop', ''))
+            if route_tuple in self.routes:
+                routes_to_be_deleted.append(route)
+
+        for route in routes_to_be_deleted:
+            _delvalue_from_list(lrouter, 'static_routes', route)
+            route.delete()
 
 
-class DelAddrSetCommand(command.BaseCommand):
-    def __init__(self, api, name, if_exists):
-        super(DelAddrSetCommand, self).__init__(api)
-        self.name = name
-        self.if_exists = if_exists
+class SetStaticRouteCommand(command.BaseCommand):
+    def __init__(self, api, sroute, **columns):
+        super().__init__(api)
+        self.sroute = sroute
+        self.columns = columns
 
     def run_idl(self, txn):
         try:
-            addrset = idlutils.row_by_value(self.api.idl, 'Address_Set',
-                                            'name', self.name)
-        except idlutils.RowNotFound:
-            if self.if_exists:
-                return
-            msg = _("Address set %s does not exist. "
-                    "Can't delete.") % self.name
-            raise RuntimeError(msg)
+            for col, val in self.columns.items():
+                setattr(self.sroute, col, val)
 
-        self.api._tables['Address_Set'].rows[addrset.uuid].delete()
+        except idlutils.RowNotFound:
+            msg = (_('Logical Router Static Route %s does not exist')
+                   % self.sroute)
+            raise RuntimeError(msg)
 
 
 class UpdateObjectExtIdsCommand(command.BaseCommand):
@@ -801,10 +966,11 @@ class CheckRevisionNumberCommand(command.BaseCommand):
 
 class DeleteLRouterExtGwCommand(command.BaseCommand):
 
-    def __init__(self, api, lrouter, if_exists):
+    def __init__(self, api, lrouter, if_exists, maintain_bfd=True):
         super(DeleteLRouterExtGwCommand, self).__init__(api)
         self.lrouter = lrouter
         self.if_exists = if_exists
+        self.maintain_bfd = maintain_bfd
 
     def run_idl(self, txn):
         try:
@@ -816,12 +982,22 @@ class DeleteLRouterExtGwCommand(command.BaseCommand):
             msg = _("Logical Router %s does not exist") % self.lrouter
             raise RuntimeError(msg)
 
+        if self.maintain_bfd:
+            lrp_names = set()
+            for lrp in getattr(lrouter, 'ports', []):
+                lrp_names.add(lrp.name)
         for route in lrouter.static_routes:
             external_ids = getattr(route, 'external_ids', {})
             if ovn_const.OVN_ROUTER_IS_EXT_GW in external_ids:
+                bfd = getattr(route, 'bfd', [])
+                if bfd and self.maintain_bfd:
+                    for bfd_rec in bfd:
+                        bfd_logical_port = getattr(bfd_rec, 'logical_port', '')
+                        if bfd_logical_port in lrp_names:
+                            route.delvalue('bfd', bfd_rec)
+                            bfd_rec.delete()
                 lrouter.delvalue('static_routes', route)
                 route.delete()
-                break
 
         for nat in lrouter.nat:
             if nat.type != 'snat':
@@ -829,19 +1005,17 @@ class DeleteLRouterExtGwCommand(command.BaseCommand):
             lrouter.delvalue('nat', nat)
             nat.delete()
 
-        lrouter_ext_ids = getattr(lrouter, 'external_ids', {})
-        gw_port_id = lrouter_ext_ids.get(ovn_const.OVN_GW_PORT_EXT_ID_KEY)
-        if not gw_port_id:
-            return
+        # Remove the router pinning to a chassis (if any).
+        lrouter.delkey('options', 'chassis')
 
-        try:
-            lrouter_port = idlutils.row_by_value(
-                self.api.idl, 'Logical_Router_Port', 'name',
-                utils.ovn_lrouter_port_name(gw_port_id))
-        except idlutils.RowNotFound:
-            return
+        # Remove the HA_Chassis_Group of the router (if any).
+        hcg = self.api.lookup('HA_Chassis_Group',
+                              lrouter.name, default=None)
+        if hcg:
+            hcg.delete()
 
-        lrouter.delvalue('ports', lrouter_port)
+        for gw_port in self.api.get_lrouter_gw_ports(lrouter.name):
+            lrouter.delvalue('ports', gw_port)
 
 
 class SetLSwitchPortToVirtualTypeCommand(command.BaseCommand):
