@@ -121,10 +121,6 @@ class TypeManager(stevedore.named.NamedExtensionManager):
                 segments, self.is_partial_segment)
             return segments
 
-    def _match_segment(self, segment, filters):
-        return all(not filters.get(attr) or segment.get(attr) in filters[attr]
-                   for attr in provider.ATTRIBUTES)
-
     def _get_provider_segment(self, network):
         # TODO(manishg): Placeholder method
         # Code intended for operating on a provider segment should use
@@ -134,18 +130,6 @@ class TypeManager(stevedore.named.NamedExtensionManager):
         # here we will do the job of extracting the segment information.
         return network
 
-    def network_matches_filters(self, network, filters):
-        if not filters:
-            return True
-        if any(validators.is_attr_set(network.get(attr))
-               for attr in provider.ATTRIBUTES):
-            segments = [self._get_provider_segment(network)]
-        elif validators.is_attr_set(network.get(mpnet_apidef.SEGMENTS)):
-            segments = self._get_attribute(network, mpnet_apidef.SEGMENTS)
-        else:
-            return True
-        return any(self._match_segment(s, filters) for s in segments)
-
     def _get_attribute(self, attrs, key):
         value = attrs.get(key)
         if value is constants.ATTR_NOT_SPECIFIED:
@@ -191,12 +175,12 @@ class TypeManager(stevedore.named.NamedExtensionManager):
         else:
             network_segments = [
                 {provider_key: network.pop(provider_key)
-                for provider_key in provider.ATTRIBUTES}]
+                 for provider_key in provider.ATTRIBUTES}]
 
         return (
             [{api.NETWORK_TYPE: network_segment[provider.NETWORK_TYPE],
-             api.PHYSICAL_NETWORK: network_segment[provider.PHYSICAL_NETWORK],
-             api.SEGMENTATION_ID: network_segment[provider.SEGMENTATION_ID]}
+              api.PHYSICAL_NETWORK: network_segment[provider.PHYSICAL_NETWORK],
+              api.SEGMENTATION_ID: network_segment[provider.SEGMENTATION_ID]}
              for network_segment in network_segments])
 
     def initialize(self):
@@ -867,7 +851,7 @@ class MechanismManager(stevedore.named.NamedExtensionManager):
                 segment = context._new_bound_segment
                 if segment:
                     pbl_obj = ports.PortBindingLevel(
-                        context._plugin_context,
+                        context.plugin_context,
                         port_id=port_id,
                         host=context.host,
                         level=level,
