@@ -1070,64 +1070,6 @@ class TestMaintenance(_TestMaintenanceHelper):
         lsp = self.nb_api.lookup('Logical_Switch_Port', p1['id'])
         self.assertEqual(hcg_uuid, lsp.ha_chassis_group[0].uuid)
 
-    def test_remove_duplicated_chassis_registers(self):
-        hostnames = ['host1', 'host2']
-        for hostname in hostnames:
-            for _ in range(3):
-                self.add_fake_chassis(hostname)
-
-        chassis = self.sb_api.chassis_list().execute(check_error=True)
-        self.assertEqual(6, len(chassis))
-        # Make the chassis private timestamp different
-        for idx, ch in enumerate(chassis):
-            self.sb_api.db_set('Chassis_Private', ch.name,
-                               ('nb_cfg_timestamp', idx)).execute()
-
-        ch_private_dict = {}  # host: [ch_private1, ch_private2, ...]
-        for hostname in hostnames:
-            ch_private_list = []
-            for ch in (ch for ch in chassis if ch.hostname == hostname):
-                ch_private = self.sb_api.lookup('Chassis_Private', ch.name,
-                                                default=None)
-                if ch_private:
-                    # One of the "Chassis_Private" has been deleted on purpose
-                    # in this test.
-                    ch_private_list.append(ch_private)
-            ch_private_list.sort(key=lambda x: x.nb_cfg_timestamp,
-                                 reverse=True)
-            ch_private_dict[hostname] = ch_private_list
-
-        self.maint.remove_duplicated_chassis_registers()
-        chassis_result = self.sb_api.chassis_list().execute(check_error=True)
-        self.assertEqual(2, len(chassis_result))
-        for ch in chassis_result:
-            self.assertIn(ch.hostname, hostnames)
-            hostnames.remove(ch.hostname)
-            # From ch_private_dict[ch.hostname], we retrieve the first
-            # "Chassis_Private" register because these are ordered by
-            # timestamp. The newer one (bigger timestamp) should remain in the
-            # system.
-            ch_expected = ch_private_dict[ch.hostname][0].chassis[0]
-            self.assertEqual(ch_expected.name, ch.name)
-
-    def test_remove_duplicated_chassis_registers_no_ch_private_register(self):
-        for _ in range(2):
-            self.add_fake_chassis('host1')
-
-        chassis = self.sb_api.chassis_list().execute(check_error=True)
-        self.assertEqual(2, len(chassis))
-        # Make the chassis private timestamp different
-        # Delete on of the "Chassis_Private" registers.
-        self.sb_api.db_destroy('Chassis_Private', chassis[0].name).execute()
-        self.sb_api.db_set('Chassis_Private', chassis[1].name,
-                           ('nb_cfg_timestamp', 1)).execute()
-
-        self.maint.remove_duplicated_chassis_registers()
-        chassis_result = self.sb_api.chassis_list().execute(check_error=True)
-        # Both "Chassis" registers are still in the DB because one
-        # "Chassis_Private" register was missing.
-        self.assertEqual(2, len(chassis_result))
-
     def _test_check_provider_distributed_ports(
             self, is_distributed_fip, net_type, expected_value=None):
         cfg.CONF.set_override(
@@ -1333,6 +1275,127 @@ class TestMaintenance(_TestMaintenanceHelper):
         lr = self.nb_api.lookup('Logical_Router', utils.ovn_name(router['id']))
         self.assertEqual([], lr.ports[0].gateway_chassis)
 
+    def test_set_network_type(self):
+        net1 = self._create_network(uuidutils.generate_uuid())
+        ls_name = utils.ovn_name(net1['id'])
+        self.nb_api.db_remove(
+            'Logical_Switch', ls_name, 'external_ids',
+            ovn_const.OVN_NETTYPE_EXT_ID_KEY).execute(check_error=True)
+        ls = self.nb_api.lookup('Logical_Switch', ls_name)
+        self.assertIsNone(ls.external_ids.get(
+            ovn_const.OVN_NETTYPE_EXT_ID_KEY))
+
+        self.assertRaises(periodics.NeverAgain, self.maint.set_network_type)
+        ls = self.nb_api.lookup('Logical_Switch', ls_name)
+        self.assertEqual(net1[provnet_apidef.NETWORK_TYPE],
+                         ls.external_ids.get(ovn_const.OVN_NETTYPE_EXT_ID_KEY))
+
+    def test_check_network_broadcast_arps_to_all_routers(self):
+        net = self._create_network('net', external=True)
+        ls = self.nb_api.get_lswitch(utils.ovn_name(net['id']))
+
+        self.assertEqual(
+            'true',
+            ls.other_config.get(ovn_const.LS_OPTIONS_BROADCAST_ARPS_ROUTERS))
+
+        # Change the value of the configuration
+        cfg.CONF.set_override(
+            'broadcast_arps_to_all_routers', False, group='ovn')
+
+        # Call the maintenance task and check that the value has been
+        # updated in the Logical Switch
+        self.assertRaises(
+            periodics.NeverAgain,
+            self.maint.check_network_broadcast_arps_to_all_routers)
+
+        ls = self.nb_api.get_lswitch(utils.ovn_name(net['id']))
+
+        self.assertEqual(
+            'false',
+            ls.other_config.get(ovn_const.LS_OPTIONS_BROADCAST_ARPS_ROUTERS))
+
+    def test_static_routes_with_external_ids(self):
+        ext_net = self._create_network('ext_networktest', external=True)
+        ext_subnet = self._create_subnet(
+            'ext_subnettest',
+            ext_net['id'],
+            **{'cidr': '100.0.0.0/24',
+               'gateway_ip': '100.0.0.254',
+               'allocation_pools': [
+                   {'start': '100.0.0.2', 'end': '100.0.0.253'}],
+               'enable_dhcp': False})
+        net1 = self._create_network('network1test', external=False)
+        subnet1 = self._create_subnet('subnet1test', net1['id'])
+        external_gateway_info = {
+            'enable_snat': True,
+            'network_id': ext_net['id'],
+            'external_fixed_ips': [
+                {'ip_address': '100.0.0.2', 'subnet_id': ext_subnet['id']}]}
+        router = self._create_router(
+            'routertest', external_gateway_info=external_gateway_info)
+        self._add_router_interface(router['id'], subnet1['id'])
+
+        # Create static routes via Neutron
+        with mock.patch.object(self.nb_api,
+                'add_static_route', columns=None):
+            self.l3_plugin.update_router(
+                self.context, router['id'],
+                {'router': {'routes': [{'destination': '10.10.0.0/24',
+                                        'nexthop': '100.0.0.3'},
+                                       {'destination': '20.0.0.0/24',
+                                        'nexthop': '100.0.0.6'}]}})
+
+        # Create a Neutron owned static route with external_ids key
+        columns = {'external_ids': {ovn_const.OVN_LRSR_EXT_ID_KEY: 'true'}}
+        with self.nb_api.transaction(check_error=True) as txn:
+            txn.add(self.nb_api.add_static_route('neutron-' + router['id'],
+                                                 ip_prefix='10.10.0.0/24',
+                                                 nexthop='100.0.0.3',
+                                                 **columns))
+
+        # Create a Neutron owned static route without external_ids key
+        with self.nb_api.transaction(check_error=True) as txn:
+            txn.add(self.nb_api.add_static_route('neutron-' + router['id'],
+                                                 ip_prefix='20.0.0.0/24',
+                                                 nexthop='100.0.0.6'))
+
+        # Create an OVN externally managed static route without external_ids
+        with self.nb_api.transaction(check_error=True) as txn:
+            txn.add(self.nb_api.add_static_route('neutron-' + router['id'],
+                                                 ip_prefix='30.0.0.0/24',
+                                                 nexthop='100.0.0.9'))
+
+        sroutes = self.nb_api.get_all_logical_routers_static_routes()[0]
+        sroute_info = sroutes['static_routes']
+        for route in sroute_info:
+            if route.ip_prefix == '10.10.0.0/24':
+                self.assertEqual(route.external_ids,
+                    {ovn_const.OVN_LRSR_EXT_ID_KEY: 'true'})
+            if route.ip_prefix == '20.0.0.0/24':
+                self.assertEqual({}, route.external_ids)
+            if route.ip_prefix == '30.0.0.0/24':
+                self.assertEqual({}, route.external_ids)
+
+        # Call the maintenance task and check that the value has been
+        # updated in the external_ids.
+        self.assertRaises(periodics.NeverAgain,
+            self.maint.update_router_static_routes)
+
+        sroutes = self.nb_api.get_all_logical_routers_static_routes()[0]
+        sroute_info = sroutes['static_routes']
+        for route in sroute_info:
+            if route.ip_prefix == '10.10.0.0/24':
+                self.assertEqual(route.external_ids,
+                    {ovn_const.OVN_LRSR_EXT_ID_KEY: 'true'})
+            # Check if the OVN static route was updated with the Neutron key
+            if route.ip_prefix == '20.0.0.0/24':
+                self.assertEqual(route.external_ids,
+                    {ovn_const.OVN_LRSR_EXT_ID_KEY: 'true'})
+            # Check if the externally managed OVN static route remains
+            # without the Neutron key.
+            if route.ip_prefix == '30.0.0.0/24':
+                self.assertEqual({}, route.external_ids)
+
     def _get_nb_global_external_ids(self):
         return self.nb_api.db_get(
             'NB_Global', '.', 'external_ids').execute(check_error=True)
@@ -1381,21 +1444,6 @@ class TestMaintenance(_TestMaintenanceHelper):
             original_value=True,
             config_value=True)
 
-    def test_set_network_type(self):
-        net1 = self._create_network(uuidutils.generate_uuid())
-        ls_name = utils.ovn_name(net1['id'])
-        self.nb_api.db_remove(
-            'Logical_Switch', ls_name, 'external_ids',
-            ovn_const.OVN_NETTYPE_EXT_ID_KEY).execute(check_error=True)
-        ls = self.nb_api.lookup('Logical_Switch', ls_name)
-        self.assertIsNone(ls.external_ids.get(
-            ovn_const.OVN_NETTYPE_EXT_ID_KEY))
-
-        self.assertRaises(periodics.NeverAgain, self.maint.set_network_type)
-        ls = self.nb_api.lookup('Logical_Switch', ls_name)
-        self.assertEqual(net1[provnet_apidef.NETWORK_TYPE],
-                         ls.external_ids.get(ovn_const.OVN_NETTYPE_EXT_ID_KEY))
-
 
 class TestLogMaintenance(_TestMaintenanceHelper,
                          test_log_driver.LogApiTestCaseBase):
