@@ -20,7 +20,7 @@ import time
 from unittest import mock
 
 import netaddr
-
+from neutron_lib.api.definitions import external_net
 from neutron_lib.api.definitions import portbindings
 from neutron_lib import constants
 from neutron_lib.exceptions import agent as agent_exc
@@ -109,13 +109,13 @@ class TestPortBinding(base.TestOVNFunctionalBase):
                 'tenant_id': self._tenant_id})
 
             port_req = self.new_create_request('ports', port_data, self.fmt,
-                                               as_admin=True)
+                                               as_service=True)
             port_res = port_req.get_response(self.api)
             p = self.deserialize(self.fmt, port_res)
             port_id = p['port']['id']
         else:
             port_req = self.new_update_request('ports', port_data, port_id,
-                                               self.fmt, as_admin=True)
+                                               self.fmt, as_service=True)
             port_res = port_req.get_response(self.api)
             self.deserialize(self.fmt, port_res)
 
@@ -635,7 +635,8 @@ class TestExternalPorts(base.TestOVNFunctionalBase):
             'ports', port_upt_data, port['id'], self.fmt)
         port_res = port_req.get_response(self.api)
 
-    def test_add_external_port_avoid_flapping(self):
+    @mock.patch('neutron.objects.router.Router.get_object')
+    def test_add_external_port_avoid_flapping(self, gr):
         class LogicalSwitchPortUpdateUpEventTest(event.RowEvent):
             def __init__(self):
                 self.count = 0
@@ -666,6 +667,7 @@ class TestExternalPorts(base.TestOVNFunctionalBase):
             def get_count(self):
                 return self.count
 
+        gr.return_value = {'flavor_id': None}
         og_up_event = ovsdb_monitor.LogicalSwitchPortUpdateUpEvent(None)
         og_down_event = ovsdb_monitor.LogicalSwitchPortUpdateDownEvent(None)
         test_down_event = LogicalSwitchPortUpdateDownEventTest()
@@ -681,7 +683,8 @@ class TestExternalPorts(base.TestOVNFunctionalBase):
         # status as up, triggering only a LogicalSwitchPortUpdateUpEvent.
         self._create_router_port(portbindings.VNIC_DIRECT)
         self.assertEqual(test_down_event.get_count(), 0)
-        self.assertEqual(test_up_event.get_count(), 1)
+        n_utils.wait_until_true(lambda: test_up_event.get_count() == 1,
+                                timeout=10)
 
     def test_external_port_create_vnic_direct(self):
         self._test_external_port_create(portbindings.VNIC_DIRECT)
@@ -737,7 +740,7 @@ class TestExternalPorts(base.TestOVNFunctionalBase):
                              ovn_const.PORT_CAP_SWITCHDEV]}}}
 
         port_req = self.new_create_request('ports', port_data, self.fmt,
-                                           as_admin=True)
+                                           as_service=True)
         port_res = port_req.get_response(self.api)
         port = self.deserialize(self.fmt, port_res)['port']
 
@@ -786,7 +789,7 @@ class TestExternalPorts(base.TestOVNFunctionalBase):
                          ovn_const.PORT_CAP_SWITCHDEV]}}}
         port_req = self.new_update_request(
             'ports', port_upt_data, port['id'], self.fmt,
-            as_admin=True)
+            as_service=True)
         port_res = port_req.get_response(self.api)
         port = self.deserialize(self.fmt, port_res)['port']
 
@@ -1138,10 +1141,9 @@ class AgentWaitEvent(event.WaitEvent):
     ONETIME = False
 
     def __init__(self, driver, chassis_names, events=None):
-        table = driver.agent_chassis_table
         events = events or (self.ROW_CREATE,)
         self.chassis_names = chassis_names
-        super().__init__(events, table, None)
+        super().__init__(events, 'Chassis_Private', None)
         self.event_name = "AgentWaitEvent"
 
     def match_fn(self, event, row, old):
@@ -1188,6 +1190,11 @@ class TestAgentApi(base.TestOVNFunctionalBase):
         agent = {'agent_type': self.TEST_AGENT, 'binary': '/bin/test',
                  'host': self.host, 'topic': 'test_topic'}
         _, status = self.plugin.create_or_update_agent(self.context, agent)
+        # Ensure the non OVN agent has been correctly created.
+        self.assertEqual(
+            status['id'],
+            self.plugin.get_agent(self.context, status['id'])['id']
+        )
         return status['id']
 
     def _check_chassis_registers(self, present=True):
@@ -1211,15 +1218,11 @@ class TestAgentApi(base.TestOVNFunctionalBase):
         agent_id = self.agent_types[ovn_const.OVN_CONTROLLER_AGENT]
         agent = self.plugin.get_agent(self.context, agent_id)
         heartbeat_timestamp = agent['heartbeat_timestamp']
-        if self.sb_api.is_table_present('Chassis_Private'):
-            chassis_ts = self.sb_api.db_get(
-                'Chassis_Private', self.chassis, 'nb_cfg_timestamp'
-            ).execute(check_error=True)
-            updated_at = datetime.datetime.fromtimestamp(
-                int(chassis_ts / 1000))
-            # if table Chassis_Private present, agent.updated_at is
-            # Chassis_Private.nb_cfg_timestamp
-            self.assertEqual(updated_at, heartbeat_timestamp)
+        chassis_ts = self.sb_api.db_get(
+            'Chassis_Private', self.chassis, 'nb_cfg_timestamp'
+        ).execute(check_error=True)
+        updated_at = datetime.datetime.fromtimestamp(int(chassis_ts / 1000))
+        self.assertEqual(updated_at, heartbeat_timestamp)
         time.sleep(1)
         # if chassis is not updated, agent's heartbeat_timestamp shouldn't
         # be updated.
@@ -1236,10 +1239,9 @@ class TestAgentApi(base.TestOVNFunctionalBase):
         # then Chassis_Private.chassis = []; both metadata and controller
         # agents will still be present in the agent list.
         agent_event = AgentWaitEvent(self.mech_driver, [self.chassis],
-                                     events=(event.RowEvent.ROW_UPDATE,
-                                             event.RowEvent.ROW_DELETE,))
+                                     events=(event.RowEvent.ROW_DELETE,))
         self.sb_api.idl.notify_handler.watch_event(agent_event)
-        self.sb_api.chassis_del(self.chassis).execute(check_error=True)
+        self.del_fake_chassis(self.chassis)
         self.assertTrue(agent_event.wait())
         agent_ids = [a['id'] for a in self.plugin.get_agents(
             self.context, filters={'host': self.host})]
@@ -1272,17 +1274,12 @@ class TestAgentApi(base.TestOVNFunctionalBase):
                           self.context, metadata_id)
 
 
-class TestNATRuleGatewayPort(base.TestOVNFunctionalBase):
+class _TestRouter(base.TestOVNFunctionalBase):
 
-    def setUp(self):
-        super().setUp()
+    def setUp(self, **kwargs):
+        super().setUp(**kwargs)
         self._ovn_client = self.mech_driver._ovn_client
 
-    def deserialize(self, content_type, response):
-        ctype = 'application/%s' % content_type
-        data = self._deserializers[ctype].deserialize(response.body)['body']
-        return data
-
     def _create_router(self, name, external_gateway_info=None):
         data = {'router': {'name': name, 'tenant_id': self._tenant_id,
                            'external_gateway_info': external_gateway_info}}
@@ -1292,6 +1289,12 @@ class TestNATRuleGatewayPort(base.TestOVNFunctionalBase):
         res = req.get_response(self.api)
         return self.deserialize(self.fmt, res)['router']
 
+    def _update_router(self, router_id, router_dict):
+        data = {'router': router_dict}
+        req = self.new_update_request('routers', data, router_id, self.fmt)
+        res = req.get_response(self.api)
+        return self.deserialize(self.fmt, res)['router']
+
     def _process_router_interface(self, action, router_id, subnet_id):
         req = self.new_action_request(
             'routers', {'subnet_id': subnet_id}, router_id,
@@ -1302,6 +1305,14 @@ class TestNATRuleGatewayPort(base.TestOVNFunctionalBase):
     def _add_router_interface(self, router_id, subnet_id):
         return self._process_router_interface('add', router_id, subnet_id)
 
+
+class TestNATRuleGatewayPort(_TestRouter):
+
+    def deserialize(self, content_type, response):
+        ctype = 'application/%s' % content_type
+        data = self._deserializers[ctype].deserialize(response.body)['body']
+        return data
+
     def _create_port(self, name, net_id, security_groups=None,
                      device_owner=None):
         data = {'port': {'name': name,
@@ -1372,3 +1383,104 @@ class TestNATRuleGatewayPort(base.TestOVNFunctionalBase):
             self.assertNotEqual([], fip_rule['gateway_port'])
         else:
             self.assertNotIn('gateway_port', fip_rule)
+
+
+class TestRouterGWPort(_TestRouter):
+
+    def _test_create_and_delete_router_gw_port(self, nested_snat=False):
+        ext_net = self._make_network(
+            self.fmt, 'ext_networktest', True, as_admin=True,
+            arg_list=('router:external',
+                      'provider:network_type',
+                      'provider:physical_network'),
+            **{'router:external': True,
+               'provider:network_type': 'flat',
+               'provider:physical_network': 'public'})['network']
+        res = self._create_subnet(self.fmt, ext_net['id'], '100.0.0.0/24')
+        ext_subnet = self.deserialize(self.fmt, res)['subnet']
+        external_gateway_info = {
+            'enable_snat': True,
+            'network_id': ext_net['id'],
+            'external_fixed_ips': [
+                {'ip_address': '100.0.0.2', 'subnet_id': ext_subnet['id']}]}
+        router = self._create_router(
+            uuidutils.generate_uuid(),
+            external_gateway_info=external_gateway_info)
+
+        inner_network = self._make_network(
+            self.fmt, 'inner_network', True)['network']
+        subnet_cidr = '192.168.0.0/24'
+        res = self._create_subnet(self.fmt, inner_network['id'],
+            '192.168.0.0/24', gateway_ip='192.168.0.1',
+            allocation_pools=[{'start': '192.168.0.2',
+                               'end': '192.168.0.253'}],
+            enable_dhcp=False)
+        inner_subnet = self.deserialize(self.fmt, res)['subnet']
+        self._add_router_interface(router['id'], inner_subnet['id'])
+
+        # Check GW LRP.
+        lr = self._ovn_client._nb_idl.lookup('Logical_Router',
+                                             utils.ovn_name(router['id']))
+
+        def _find_ext_gw_lrp(lr):
+            for lrp in lr.ports:
+                if (lrp.external_ids[ovn_const.OVN_ROUTER_IS_EXT_GW] ==
+                        str(True)):
+                    return lrp
+
+        self.assertIsNotNone(_find_ext_gw_lrp(lr))
+
+        nats = lr.nat
+        self.assertEqual(1, len(nats))
+        expected_logical_ip = (
+            ovn_const.OVN_DEFAULT_SNAT_CIDR if nested_snat else subnet_cidr
+        )
+        self.assertEqual(expected_logical_ip, nats[0].logical_ip)
+
+        # Remove LR GW port and check.
+        self._update_router(router['id'], {'external_gateway_info': {}})
+        lr = self._ovn_client._nb_idl.lookup('Logical_Router',
+                                             utils.ovn_name(router['id']))
+        self.assertEqual([], lr.nat)
+        self.assertIsNone(_find_ext_gw_lrp(lr))
+
+    def test_create_and_delete_router_gw_port(self):
+        self._test_create_and_delete_router_gw_port()
+
+    def test_create_and_delete_router_gw_port_nested_snat(self):
+        ovn_conf.cfg.CONF.set_override('ovn_router_indirect_snat', True, 'ovn')
+        self._test_create_and_delete_router_gw_port(nested_snat=True)
+
+
+class TestHAChassisGroupSync(base.TestOVNFunctionalBase):
+    def test_ha_chassis_keep_priority_external_ports(self):
+        for i in range(5):
+            self.add_fake_chassis('host%d' % i, enable_chassis_as_gw=True,
+                                  azs=[])
+        net_arg = {external_net.EXTERNAL: True}
+        with self.network('test-ovn-client', as_admin=True,
+                          arg_list=tuple(net_arg.keys()), **net_arg) as net:
+            with self.subnet(net) as subnet:
+                port_arg = {
+                    portbindings.VNIC_TYPE: portbindings.VNIC_BAREMETAL}
+                with self.port(subnet,
+                               arg_list=tuple(port_arg.keys()), **port_arg):
+                    hcg1 = self.mech_driver._ovn_client._nb_idl.lookup(
+                        'HA_Chassis_Group',
+                        utils.ovn_name(net['network']['id']))
+                    self.assertIsNotNone(hcg1)
+                    master_ch1 = max(hcg1.ha_chassis, key=lambda x: x.priority,
+                                     default=None)
+                    with self.port(subnet, arg_list=tuple(port_arg.keys()),
+                                   **port_arg):
+                        hcg2 = self.mech_driver._ovn_client._nb_idl.lookup(
+                            'HA_Chassis_Group',
+                            utils.ovn_name(net['network']['id']))
+                        master_ch2 = max(hcg2.ha_chassis,
+                                         key=lambda x: x.priority,
+                                         default=None)
+                        # A second port created on this network will have the
+                        # same HCG assigned and the same highest priority
+                        # chassis.
+                        self.assertEqual(master_ch1.chassis_name,
+                                         master_ch2.chassis_name)
