@@ -219,13 +219,13 @@ class TestUtils(base.BaseTestCase):
     def test_is_ovn_metadata_port(self):
         meta_port = {
             'device_owner': n_const.DEVICE_OWNER_DISTRIBUTED,
-            'device_id': 'ovnmeta-12345'}
+            'device_id': constants.OVN_METADATA_PREFIX + '12345'}
         non_meta_port_0 = {
             'device_owner': n_const.DEVICE_OWNER_DISTRIBUTED,
             'device_id': 'dhcp-12345'}
         non_meta_port_1 = {
             'device_owner': n_const.DEVICE_OWNER_DHCP,
-            'device_id': 'ovnmeta-12345'}
+            'device_id': constants.OVN_METADATA_PREFIX + '12345'}
 
         self.assertTrue(utils.is_ovn_metadata_port(meta_port))
         self.assertFalse(utils.is_ovn_metadata_port(non_meta_port_0))
@@ -237,7 +237,7 @@ class TestUtils(base.BaseTestCase):
             'device_id': 'ovn-lb-hm-12345'}
         non_ovn_lb_hm_port_0 = {
             'device_owner': n_const.DEVICE_OWNER_DISTRIBUTED,
-            'device_id': 'ovnmeta-12345'}
+            'device_id': constants.OVN_METADATA_PREFIX + '12345'}
         non_ovn_lb_hm_port_1 = {
             'device_owner': n_const.DEVICE_OWNER_DHCP,
             'device_id': 'dhcp-12345'}
@@ -266,7 +266,7 @@ class TestGateWayChassisValidity(base.BaseTestCase):
 
     def test_gateway_chassis_due_to_invalid_chassis_name(self):
         # Return True since chassis is invalid
-        self.chassis_name = constants.OVN_GATEWAY_INVALID_CHASSIS
+        self.chassis_name = None
         self.assertTrue(utils.is_gateway_chassis_invalid(
             self.chassis_name, self.gw_chassis, self.physnet,
             self.chassis_physnets, self.az_hints, self.chassis_azs))
@@ -439,17 +439,29 @@ class TestDHCPUtils(base.BaseTestCase):
         self.assertEqual(expected_options, options)
 
     def test_get_lsp_dhcp_opts_for_baremetal(self):
-        opt0 = {'opt_name': 'tag:ipxe,bootfile-name',
-                'opt_value': 'http://172.7.27.29/ipxe',
-                'ip_version': 4}
-        opt1 = {'opt_name': 'tag:!ipxe,bootfile-name',
-                'opt_value': 'undionly.kpxe',
-                'ip_version': 4}
-        opt2 = {'opt_name': 'tftp-server',
-                'opt_value': '"172.7.27.29"',
-                'ip_version': 4}
+        opts = [{
+            'opt_name': 'tag:ipxe,bootfile-name',
+            'opt_value': 'http://172.7.27.29/ipxe',
+            'ip_version': 4
+        }, {
+            'opt_name': 'tag:!ipxe,bootfile-name',
+            'opt_value': 'undionly.kpxe',
+            'ip_version': 4
+        }, {
+            'opt_name': 'tftp-server',
+            'opt_value': '"172.7.27.29"',
+            'ip_version': 4
+        }, {
+            'opt_name': 'tag:ipxe6,bootfile-name',
+            'opt_value': 'http://[2001:db8::1]/ipxe',
+            'ip_version': 6
+        }, {
+            'opt_name': 'tag:!ipxe6,bootfile-name',
+            'opt_value': 'undionly.kpxe',
+            'ip_version': 6
+        }]
         port = {portbindings.VNIC_TYPE: portbindings.VNIC_BAREMETAL,
-                edo_ext.EXTRADHCPOPTS: [opt0, opt1, opt2]}
+                edo_ext.EXTRADHCPOPTS: opts}
 
         dhcp_disabled, options = utils.get_lsp_dhcp_opts(port, 4)
         self.assertFalse(dhcp_disabled)
@@ -459,16 +471,28 @@ class TestDHCPUtils(base.BaseTestCase):
                             'bootfile_name': '"http://172.7.27.29/ipxe"',
                             'bootfile_name_alt': '"undionly.kpxe"'}
         self.assertEqual(expected_options, options)
+        # Now the same for IPv6 options
+        dhcp_disabled, options = utils.get_lsp_dhcp_opts(port, 6)
+        self.assertFalse(dhcp_disabled)
+        expected_options = {'bootfile_name': '"http://[2001:db8::1]/ipxe"',
+                            'bootfile_name_alt': '"undionly.kpxe"'}
+        self.assertEqual(expected_options, options)
 
     def test_get_lsp_dhcp_opts_dhcp_disabled_for_baremetal(self):
         cfg.CONF.set_override(
             'disable_ovn_dhcp_for_baremetal_ports', True, group='ovn')
 
-        opt = {'opt_name': 'tag:ipxe,bootfile-name',
-               'opt_value': 'http://172.7.27.29/ipxe',
-               'ip_version': 4}
+        opts = [{
+            'opt_name': 'tag:ipxe,bootfile-name',
+            'opt_value': 'http://172.7.27.29/ipxe',
+            'ip_version': 4
+        }, {
+            'opt_name': 'tag:ipxe,bootfile-name',
+            'opt_value': 'http://[2001:db8::1]/ipxe',
+            'ip_version': 6
+        }]
         port = {portbindings.VNIC_TYPE: portbindings.VNIC_BAREMETAL,
-                edo_ext.EXTRADHCPOPTS: [opt]}
+                edo_ext.EXTRADHCPOPTS: [opts]}
 
         dhcp_disabled, options = utils.get_lsp_dhcp_opts(port, 4)
         # Assert DHCP is disabled for this port
@@ -476,6 +500,13 @@ class TestDHCPUtils(base.BaseTestCase):
         # Assert no options were passed
         self.assertEqual({}, options)
 
+        # and the same for dhcpv6
+        dhcp_disabled, options = utils.get_lsp_dhcp_opts(port, 6)
+        # Assert DHCP is disabled for this port
+        self.assertTrue(dhcp_disabled)
+        # Assert no options were passed
+        self.assertEqual({}, options)
+
     def test_get_lsp_dhcp_opts_for_domain_search(self):
         opt = {'opt_name': 'domain-search',
                'opt_value': 'openstack.org,ovn.org',
@@ -575,6 +606,7 @@ class TestValidateAndGetDataFromBindingProfile(base.BaseTestCase):
             'neutron_lib.plugins.directory.get_plugin').start()
         self.VNIC_FAKE_NORMAL = 'fake-vnic-normal'
         self.VNIC_FAKE_OTHER = 'fake-vnic-other'
+        self.VNIC_FAKE_THIRD = 'fake-vnic-third'
 
         # Replace constants.OVN_PORT_BINDING_PROFILE_PARAMS to allow synthesis
         _params = constants.OVN_PORT_BINDING_PROFILE_PARAMS.copy()
@@ -582,9 +614,6 @@ class TestValidateAndGetDataFromBindingProfile(base.BaseTestCase):
             constants.OVNPortBindingProfileParamSet(
                 {'key': [str, type(None)]},
                 self.VNIC_FAKE_NORMAL, None),
-            constants.OVNPortBindingProfileParamSet(
-                {'key': [str], 'other_key': [str]},
-                self.VNIC_FAKE_OTHER, None),
             constants.OVNPortBindingProfileParamSet(
                 {
                     'key': [str],
@@ -592,6 +621,9 @@ class TestValidateAndGetDataFromBindingProfile(base.BaseTestCase):
                     'third_key': [str]
                 },
                 self.VNIC_FAKE_OTHER, constants.PORT_CAP_SWITCHDEV),
+            constants.OVNPortBindingProfileParamSet(
+                {'key': [str], 'other_key': [str]},
+                self.VNIC_FAKE_THIRD, None),
         ])
         self.OVN_PORT_BINDING_PROFILE_PARAMS = mock.patch.object(
             constants,
@@ -706,6 +738,27 @@ class TestValidateAndGetDataFromBindingProfile(base.BaseTestCase):
                 {portbindings.VNIC_TYPE: portbindings.VNIC_REMOTE_MANAGED,
                  constants.OVN_PORT_BINDING_PROFILE: expect}))
 
+    def test_valid_input_surplus_capabilities(self):
+        capabilities = ['rx', 'tx', 'sg', 'tso', 'gso', 'gro', 'rxvlan',
+                        'txvlan', 'rxhash', 'rdma', 'txudptnl']
+        binding_profile = {
+            'pci_vendor_info': 'dead:beef',
+            'pci_slot': '0000:ca:fe.42',
+            'physical_network': 'physnet1',
+            'card_serial_number': 'AB2000X00042',
+            'pf_mac_address': '00:53:00:00:00:42',
+            'vf_num': 42,
+            constants.PORT_CAP_PARAM: capabilities
+        }
+        expect = binding_profile.copy()
+        del(expect[constants.PORT_CAP_PARAM])
+        self.assertEqual(
+            utils.BPInfo(expect, portbindings.VNIC_REMOTE_MANAGED,
+                         capabilities),
+            utils.validate_and_get_data_from_binding_profile(
+                {portbindings.VNIC_TYPE: portbindings.VNIC_REMOTE_MANAGED,
+                 constants.OVN_PORT_BINDING_PROFILE: binding_profile}))
+
     def test_valid_input_surplus_keys(self):
         # Confirm that extra keys are allowed
         binding_profile = {
@@ -779,12 +832,12 @@ class TestValidateAndGetDataFromBindingProfile(base.BaseTestCase):
             utils.validate_and_get_data_from_binding_profile(
                 {portbindings.VNIC_TYPE: self.VNIC_FAKE_NORMAL,
                  constants.OVN_PORT_BINDING_PROFILE: binding_profile}))
-        # It is valid for VNIC_FAKE_OTHER
+        # It is valid for VNIC_FAKE_THIRD
         expected_bp = binding_profile.copy()
         self.assertEqual(
-            utils.BPInfo(expected_bp, self.VNIC_FAKE_OTHER, []),
+            utils.BPInfo(expected_bp, self.VNIC_FAKE_THIRD, []),
             utils.validate_and_get_data_from_binding_profile(
-                {portbindings.VNIC_TYPE: self.VNIC_FAKE_OTHER,
+                {portbindings.VNIC_TYPE: self.VNIC_FAKE_THIRD,
                  constants.OVN_PORT_BINDING_PROFILE: binding_profile}))
 
     def test_overlapping_param_set_different_vnic_type_and_capability(self):
@@ -794,13 +847,13 @@ class TestValidateAndGetDataFromBindingProfile(base.BaseTestCase):
             'other_key': 42,
             'third_key': 'value',
         }
-        # This param set is not valid for VNIC_FAKE_OTHER without capability
+        # This param set is not valid for VNIC_FAKE_THIRD without capability
         expect = binding_profile.copy()
         del(expect['third_key'])
         self.assertRaises(
             neutron_lib.exceptions.InvalidInput,
             utils.validate_and_get_data_from_binding_profile,
-            {portbindings.VNIC_TYPE: self.VNIC_FAKE_OTHER,
+            {portbindings.VNIC_TYPE: self.VNIC_FAKE_THIRD,
              constants.OVN_PORT_BINDING_PROFILE: binding_profile})
         # This param set is also not valid as the capabilities do not match
         binding_profile = {
@@ -1015,99 +1068,99 @@ class GetSubnetsAddressScopeTestCase(base.BaseTestCase):
         self.ml2_plugin = mock.Mock()
 
     def test_no_subnets(self):
-        subnets = []
+        subnets_by_id = []
         fixed_ips = mock.ANY
         address4, address6 = utils.get_subnets_address_scopes(
-            mock.ANY, subnets, fixed_ips, self.ml2_plugin)
+            mock.ANY, subnets_by_id, fixed_ips, self.ml2_plugin)
         self.assertEqual(('', ''), (address4, address6))
 
     def test_no_subnetpool(self):
-        subnets = [
-            {'id': 'subnet1', 'subnetpool_id': None},
-            {'id': 'subnet2', 'subnetpool_id': None},
-        ]
+        subnets_by_id = {
+            'subnet_id1': {'name': 'subnet1', 'subnetpool_id': None},
+            'subnet_id2': {'name': 'subnet2', 'subnetpool_id': None},
+        }
         fixed_ips = [
-            {'subnet_id': 'subnet1'},
-            {'subnet_id': 'subnet2'},
+            {'subnet_id': 'subnet_id1'},
+            {'subnet_id': 'subnet_id2'},
         ]
         address4, address6 = utils.get_subnets_address_scopes(
-            mock.ANY, subnets, fixed_ips, self.ml2_plugin)
+            mock.ANY, subnets_by_id, fixed_ips, self.ml2_plugin)
         self.assertEqual(('', ''), (address4, address6))
 
     def test_no_address_scope(self):
-        subnets = [
-            {'id': 'subnet1', 'subnetpool_id': 'pool_ipv4'},
-            {'id': 'subnet2', 'subnetpool_id': 'pool_ipv6'},
-        ]
+        subnets_by_id = {
+            'subnet_id1': {'name': 'subnet1', 'subnetpool_id': 'pool_ipv4'},
+            'subnet_id2': {'name': 'subnet2', 'subnetpool_id': 'pool_ipv6'},
+        }
         fixed_ips = [
-            {'subnet_id': 'subnet1'},
-            {'subnet_id': 'subnet2'},
+            {'subnet_id': 'subnet_id1'},
+            {'subnet_id': 'subnet_id2'},
         ]
         self.ml2_plugin.get_subnetpool.side_effect = n_exc.SubnetPoolNotFound(
             subnetpool_id='snp')
         address4, address6 = utils.get_subnets_address_scopes(
-            mock.ANY, subnets, fixed_ips, self.ml2_plugin)
+            mock.ANY, subnets_by_id, fixed_ips, self.ml2_plugin)
         self.assertEqual(('', ''), (address4, address6))
 
     def test_address_scope(self):
-        subnets = [
-            {'id': 'subnet1', 'subnetpool_id': 'pool_ipv4'},
-            {'id': 'subnet2', 'subnetpool_id': 'pool_ipv6'},
-        ]
+        subnets_by_id = {
+            'subnet_id1': {'name': 'subnet1', 'subnetpool_id': 'pool_ipv4'},
+            'subnet_id2': {'name': 'subnet2', 'subnetpool_id': 'pool_ipv6'},
+        }
         fixed_ips = [
-            {'subnet_id': 'subnet1'},
-            {'subnet_id': 'subnet2'},
+            {'subnet_id': 'subnet_id1'},
+            {'subnet_id': 'subnet_id2'},
         ]
         self.ml2_plugin.get_subnetpool.side_effect = [
             {'address_scope_id': 'scope4', 'ip_version': n_const.IP_VERSION_4},
             {'address_scope_id': 'scope6', 'ip_version': n_const.IP_VERSION_6},
         ]
         address4, address6 = utils.get_subnets_address_scopes(
-            mock.ANY, subnets, fixed_ips, self.ml2_plugin)
+            mock.ANY, subnets_by_id, fixed_ips, self.ml2_plugin)
         self.assertEqual(('scope4', 'scope6'), (address4, address6))
 
 
 class GetPortTypeVirtualAndParentsTestCase(base.BaseTestCase):
 
     def test_no_subnets(self):
-        subnets = []
+        subnets_by_id = []
         fixed_ips = []
         port_type, virtual_ip, virtual_parents = (
-            utils.get_port_type_virtual_and_parents(subnets, fixed_ips, 'net1',
-                                                    'port1', mock.ANY))
+            utils.get_port_type_virtual_and_parents(subnets_by_id, fixed_ips,
+                                                    'net1', 'port1', mock.ANY))
         self.assertEqual(('', None, None),
                          (port_type, virtual_ip, virtual_parents))
 
     @mock.patch.object(utils, 'get_virtual_port_parents', return_value=[])
     def test_no_parents(self, *args):
-        subnets = [
-            {'id': 'subnet1'},
-            {'id': 'subnet2'},
-        ]
+        subnets_by_id = {
+            'subnet_id1': {'name': 'subnet1'},
+            'subnet_id2': {'name': 'subnet2'},
+        }
         fixed_ips = [
-            {'subnet_id': 'subnet1', 'ip_address': '1.2.3.4'},
-            {'subnet_id': 'subnet2', 'ip_address': '1.2.3.5'},
+            {'subnet_id': 'subnet_id1', 'ip_address': '1.2.3.4'},
+            {'subnet_id': 'subnet_id2', 'ip_address': '1.2.3.5'},
         ]
         port_type, virtual_ip, virtual_parents = (
-            utils.get_port_type_virtual_and_parents(subnets, fixed_ips, 'net1',
-                                                    'port1', mock.ANY))
+            utils.get_port_type_virtual_and_parents(subnets_by_id, fixed_ips,
+                                                    'net1', 'port1', mock.ANY))
         self.assertEqual(('', None, None),
                          (port_type, virtual_ip, virtual_parents))
 
     @mock.patch.object(utils, 'get_virtual_port_parents',
                        return_value=['parent1', 'parent2'])
     def test_with_parents(self, *args):
-        subnets = [
-            {'id': 'subnet1'},
-            {'id': 'subnet2'},
-        ]
+        subnets_by_id = {
+            'subnet_id1': {'name': 'subnet1'},
+            'subnet_id2': {'name': 'subnet2'},
+        }
         fixed_ips = [
-            {'subnet_id': 'subnet1', 'ip_address': '1.2.3.4'},
-            {'subnet_id': 'subnet2', 'ip_address': '1.2.3.5'},
+            {'subnet_id': 'subnet_id1', 'ip_address': '1.2.3.4'},
+            {'subnet_id': 'subnet_id2', 'ip_address': '1.2.3.5'},
         ]
         port_type, virtual_ip, virtual_parents = (
-            utils.get_port_type_virtual_and_parents(subnets, fixed_ips, 'net1',
-                                                    'port1', mock.ANY))
+            utils.get_port_type_virtual_and_parents(subnets_by_id, fixed_ips,
+                                                    'net1', 'port1', mock.ANY))
         self.assertEqual((constants.LSP_TYPE_VIRTUAL, '1.2.3.4',
                           'parent1,parent2'),
                          (port_type, virtual_ip, virtual_parents))
