@@ -23,6 +23,7 @@ import eventlet
 from eventlet import queue
 import netaddr
 from neutron_lib import constants
+from oslo_config import cfg
 from oslo_log import log as logging
 from osprofiler import profiler
 import testscenarios
@@ -178,12 +179,6 @@ class TestDvrServices(base.BaseTestCase):
     def _test_is_dvr_serviced(self, device_owner, expected):
         self.assertEqual(expected, utils.is_dvr_serviced(device_owner))
 
-    def test_is_dvr_serviced_with_lb_port(self):
-        self._test_is_dvr_serviced(constants.DEVICE_OWNER_LOADBALANCER, True)
-
-    def test_is_dvr_serviced_with_lbv2_port(self):
-        self._test_is_dvr_serviced(constants.DEVICE_OWNER_LOADBALANCERV2, True)
-
     def test_is_dvr_serviced_with_dhcp_port(self):
         self._test_is_dvr_serviced(constants.DEVICE_OWNER_DHCP, True)
 
@@ -196,12 +191,6 @@ class TestFipServices(base.BaseTestCase):
     def _test_is_fip_serviced(self, device_owner, expected):
         self.assertEqual(expected, utils.is_fip_serviced(device_owner))
 
-    def test_is_fip_serviced_with_lb_port(self):
-        self._test_is_fip_serviced(constants.DEVICE_OWNER_LOADBALANCER, True)
-
-    def test_is_fip_serviced_with_lbv2_port(self):
-        self._test_is_fip_serviced(constants.DEVICE_OWNER_LOADBALANCERV2, True)
-
     def test_is_fip_serviced_with_dhcp_port(self):
         self._test_is_fip_serviced(constants.DEVICE_OWNER_DHCP, False)
 
@@ -329,7 +318,7 @@ class TestPortRuleMasking(base.BaseTestCase):
     def compare_port_ranges_results(self, port_min, port_max):
         observed = utils.port_rule_masking(port_min, port_max)
         expected = _port_rule_masking(port_min, port_max)
-        self.assertItemsEqual(expected, observed)
+        self.assertCountEqual(expected, observed)
 
     def test_port_rule_masking_random_ranges(self):
         # calling randint a bunch of times is really slow
@@ -425,7 +414,7 @@ class TestThrottler(base.BaseTestCase):
 
         def sleep_mock(amount_to_sleep):
             sleep(amount_to_sleep)
-            self.assertGreater(threshold, amount_to_sleep)
+            self.assertGreaterEqual(threshold, amount_to_sleep)
 
         with mock.patch.object(utils.eventlet, "sleep",
                                side_effect=sleep_mock):
@@ -588,40 +577,124 @@ class SpawnWithOrWithoutProfilerTestCase(
 
 
 @utils.SingletonDecorator
-class _TestSingletonClass(object):
+class _TestSingletonClass1():
+    def __init__(self, variable):
+        self.variable = variable
 
-    def __init__(self):
-        self.variable = None
+
+@utils.SingletonDecorator
+class _TestSingletonClass2():
+    def __init__(self, variable):
+        self.variable = variable
 
 
 class SingletonDecoratorTestCase(base.BaseTestCase):
+    def setUp(self):
+        super().setUp()
+        self.instance_1 = self.instance_2 = None
+        self.addCleanup(self._destroy_instances)
+
+    def _destroy_instances(self):
+        try:
+            del self.instance_1
+        except AttributeError:
+            pass
+        try:
+            del self.instance_2
+        except AttributeError:
+            pass
 
     def test_singleton_instance_class(self):
-        instance_1 = _TestSingletonClass()
-        instance_1.variable = 'value1'
+        self.instance_1 = _TestSingletonClass1('value1')
+        self.instance_2 = _TestSingletonClass1('other_value_that_is_not_set')
+        self.assertIs(self.instance_1, self.instance_2)
+        self.assertEqual('value1', self.instance_2.variable)
 
-        instance_2 = _TestSingletonClass()
-        self.assertEqual(instance_1.__hash__(), instance_2.__hash__())
-        self.assertEqual('value1', instance_2.variable)
+        # If one reference is deleted, the other variable (instance_2) is still
+        # referenced to the same object and the singleton decorator keeps it.
+        del self.instance_1
+        self.instance_1 = _TestSingletonClass1('again_value_that_is_not_set')
+        self.assertIs(self.instance_1, self.instance_2)
+        self.assertEqual('value1', self.instance_2.variable)
+
+    def test_singleton_several_independent_classes(self):
+        self.instance_1 = _TestSingletonClass1('value1')
+        self.instance_2 = _TestSingletonClass2('value2')
+        self.assertIsNot(self.instance_1, self.instance_2)
+        self.assertEqual('value1', self.instance_1.variable)
+        self.assertEqual('value2', self.instance_2.variable)
+
+    def test_singleton_instance_deletion(self):
+        self.instance_1 = _TestSingletonClass1('value1')
+        self.instance_2 = _TestSingletonClass2('value2')
+        self.assertIsNotNone(self.instance_1)
+        self.assertIsNotNone(self.instance_2)
+
+        # Delete instance 1, new value "value1_b" set in __init__()
+        del self.instance_1
+        self.instance_1 = _TestSingletonClass1('value1_b')
+        self.instance_2 = _TestSingletonClass2('value2_b')
+        self.assertIsNotNone(self.instance_1)
+        self.assertIsNotNone(self.instance_2)
+        self.assertEqual('value1_b', self.instance_1.variable)
+        self.assertEqual('value2', self.instance_2.variable)
+
+        # Delete instance 2, new value "value2_c" set in __init__()
+        del self.instance_2
+        self.instance_1 = _TestSingletonClass1('value1_c')
+        self.instance_2 = _TestSingletonClass2('value2_c')
+        self.assertIsNotNone(self.instance_1)
+        self.assertIsNotNone(self.instance_2)
+        self.assertEqual('value1_b', self.instance_1.variable)
+        self.assertEqual('value2_c', self.instance_2.variable)
 
 
-class SkipDecoratorTestCase(base.BaseTestCase):
+class DisableNotificationTestCase(base.BaseTestCase):
 
-    def test_skip_exception(self):
-        @utils.skip_exceptions(AttributeError)
-        def raise_attribute_error_single_exception():
-            raise AttributeError()
+    @utils.disable_notifications
+    def sample_method(self):
+        raise AttributeError()
 
-        @utils.skip_exceptions([AttributeError, IndexError])
-        def raise_attribute_error_exception_list():
-            raise AttributeError()
+    def test_notification_rpc_workers_lt_one(self):
+        cfg.CONF.set_override('rpc_workers', 0)
+        self.assertIsNone(self.sample_method())
 
-        raise_attribute_error_single_exception()
-        raise_attribute_error_exception_list()
+    def test_notification_rpc_workers_none(self):
+        cfg.CONF.set_override('rpc_workers', None)
+        self.assertRaises(AttributeError, self.sample_method)
 
-    def test_skip_exception_fail(self):
-        @utils.skip_exceptions(IndexError)
-        def raise_attribute_error():
-            raise AttributeError()
+    def test_notification_rpc_workers_one(self):
+        cfg.CONF.set_override('rpc_workers', 1)
+        self.assertRaises(AttributeError, self.sample_method)
 
-        self.assertRaises(AttributeError, raise_attribute_error)
+
+class SignatureTestCase(base.BaseTestCase):
+
+    def test_sign_instance_id(self):
+        conf = mock.Mock()
+        conf.metadata_proxy_shared_secret = 'secret'
+        self.assertEqual(
+            '773ba44693c7553d6ee20f61ea5d2757a9a4f4a44d2841ae4e95b52e4cd62db4',
+            utils.sign_instance_id(conf, 'foo')
+        )
+
+
+class ParsePermittedEthertypesTestCase(base.BaseTestCase):
+
+    @mock.patch.object(utils, 'LOG')
+    def test_parse_permitted_ethertypes(self, mock_log):
+        permitted_ethertypes = ['0x1234',
+                                '0x1234',
+                                '0x5678',
+                                '0XCAfe',
+                                '1112',
+                                '0x123R']
+        ret = utils.parse_permitted_ethertypes(permitted_ethertypes)
+        self.assertEqual({'0x1234', '0x5678', '0xcafe'}, ret)
+        calls = [mock.call('Custom ethertype %s is repeated', '0x1234'),
+                 mock.call('Custom ethertype %s is not a hexadecimal number.',
+                           '1112'),
+                 mock.call('Custom ethertype %s is not a hexadecimal number.',
+                           '0x123R'),
+                 ]
+        mock_log.warning.assert_has_calls(calls)
