@@ -16,8 +16,9 @@
 
 from unittest import mock
 
-import neutron.plugins.ml2.drivers.openvswitch.agent.common.constants \
-    as ovs_const
+from neutron_lib import constants as lib_constants
+from neutron_lib.plugins.ml2 import ovs_constants as ovs_const
+
 from neutron.tests.unit.plugins.ml2.drivers.openvswitch.agent.openflow.native \
     import ovs_bridge_test_base
 
@@ -48,11 +49,81 @@ class OVSTunnelBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase,
         self.setup_bridge_mock('br-tun', self.br_tun_cls)
         self.stamp = self.br.default_cookie
 
+    def _get_learn_flows(self, ofpp, patch_int_ofport):
+        (dp, ofp, ofpp) = self._get_dp()
+        # flows_data is list of tuples (priority, match)
+        flows_data = [
+            (2, ofpp.OFPMatch(
+                eth_type=self.ether_types.ETH_TYPE_ARP,
+                arp_tha=lib_constants.BROADCAST_MAC
+            )),
+            (2, ofpp.OFPMatch(
+                eth_type=self.ether_types.ETH_TYPE_IPV6,
+                ip_proto=self.in_proto.IPPROTO_ICMPV6,
+                icmpv6_type=self.icmpv6.ND_ROUTER_ADVERT
+            )),
+            (2, ofpp.OFPMatch(
+                eth_type=self.ether_types.ETH_TYPE_IPV6,
+                ip_proto=self.in_proto.IPPROTO_ICMPV6,
+                icmpv6_type=self.icmpv6.ND_NEIGHBOR_ADVERT
+            )),
+            (1, ofpp.OFPMatch())
+        ]
+        learn_flows = []
+        for priority, match in flows_data:
+            learn_flows.append(
+                call._send_msg(
+                    ofpp.OFPFlowMod(
+                        dp,
+                        cookie=self.stamp,
+                        instructions=[
+                            ofpp.OFPInstructionActions(
+                                ofp.OFPIT_APPLY_ACTIONS, [
+                                    ofpp.NXActionLearn(
+                                        cookie=self.stamp,
+                                        hard_timeout=300,
+                                        priority=1,
+                                        specs=[
+                                            ofpp.NXFlowSpecMatch(
+                                                dst=('vlan_tci', 0),
+                                                n_bits=12,
+                                                src=('vlan_tci', 0)),
+                                            ofpp.NXFlowSpecMatch(
+                                                dst=('eth_dst', 0),
+                                                n_bits=48,
+                                                src=('eth_src', 0)),
+                                            ofpp.NXFlowSpecLoad(
+                                                dst=('vlan_tci', 0),
+                                                n_bits=16,
+                                                src=0),
+                                            ofpp.NXFlowSpecLoad(
+                                                dst=('tunnel_id', 0),
+                                                n_bits=64,
+                                                src=('tunnel_id', 0)),
+                                            ofpp.NXFlowSpecOutput(
+                                                dst='',
+                                                n_bits=32,
+                                                src=('in_port', 0)),
+                                        ],
+                                        table_id=20),
+                                    ofpp.OFPActionOutput(patch_int_ofport, 0),
+                                ]
+                            ),
+                        ],
+                        match=match,
+                        priority=priority,
+                        table_id=10),
+                    active_bundle=None
+                )
+            )
+        return learn_flows
+
     def test_setup_default_table(self):
         patch_int_ofport = 5555
         arp_responder_enabled = False
         self.br.setup_default_table(patch_int_ofport=patch_int_ofport,
-            arp_responder_enabled=arp_responder_enabled)
+            arp_responder_enabled=arp_responder_enabled,
+            dvr_enabled=False)
         (dp, ofp, ofpp) = self._get_dp()
         expected = [
             call._send_msg(ofpp.OFPFlowMod(dp,
@@ -100,45 +171,9 @@ class OVSTunnelBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase,
                 instructions=[],
                 match=ofpp.OFPMatch(),
                 priority=0, table_id=6),
-                           active_bundle=None),
-            call._send_msg(ofpp.OFPFlowMod(dp,
-                cookie=self.stamp,
-                instructions=[
-                    ofpp.OFPInstructionActions(ofp.OFPIT_APPLY_ACTIONS, [
-                        ofpp.NXActionLearn(
-                            cookie=self.stamp,
-                            hard_timeout=300,
-                            priority=1,
-                            specs=[
-                                ofpp.NXFlowSpecMatch(
-                                    dst=('vlan_tci', 0),
-                                    n_bits=12,
-                                    src=('vlan_tci', 0)),
-                                ofpp.NXFlowSpecMatch(
-                                    dst=('eth_dst', 0),
-                                    n_bits=48,
-                                    src=('eth_src', 0)),
-                                ofpp.NXFlowSpecLoad(
-                                    dst=('vlan_tci', 0),
-                                    n_bits=16,
-                                    src=0),
-                                ofpp.NXFlowSpecLoad(
-                                    dst=('tunnel_id', 0),
-                                    n_bits=64,
-                                    src=('tunnel_id', 0)),
-                                ofpp.NXFlowSpecOutput(
-                                    dst='',
-                                    n_bits=32,
-                                    src=('in_port', 0)),
-                            ],
-                            table_id=20),
-                        ofpp.OFPActionOutput(patch_int_ofport, 0),
-                    ]),
-                ],
-                match=ofpp.OFPMatch(),
-                priority=1,
-                table_id=10),
-                           active_bundle=None),
+                           active_bundle=None)]
+        expected += self._get_learn_flows(ofpp, patch_int_ofport)
+        expected += [
             call._send_msg(ofpp.OFPFlowMod(dp,
                 cookie=self.stamp,
                 instructions=[ofpp.OFPInstructionGotoTable(table_id=22)],
@@ -160,7 +195,8 @@ class OVSTunnelBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase,
         patch_int_ofport = 5555
         arp_responder_enabled = True
         self.br.setup_default_table(patch_int_ofport=patch_int_ofport,
-            arp_responder_enabled=arp_responder_enabled)
+            arp_responder_enabled=arp_responder_enabled,
+            dvr_enabled=False)
         (dp, ofp, ofpp) = self._get_dp()
         expected = [
             call._send_msg(ofpp.OFPFlowMod(dp,
@@ -217,45 +253,9 @@ class OVSTunnelBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase,
                 instructions=[],
                 match=ofpp.OFPMatch(),
                 priority=0, table_id=6),
-                           active_bundle=None),
-            call._send_msg(ofpp.OFPFlowMod(dp,
-                cookie=self.stamp,
-                instructions=[
-                    ofpp.OFPInstructionActions(ofp.OFPIT_APPLY_ACTIONS, [
-                        ofpp.NXActionLearn(
-                            cookie=self.stamp,
-                            hard_timeout=300,
-                            priority=1,
-                            specs=[
-                                ofpp.NXFlowSpecMatch(
-                                    dst=('vlan_tci', 0),
-                                    n_bits=12,
-                                    src=('vlan_tci', 0)),
-                                ofpp.NXFlowSpecMatch(
-                                    dst=('eth_dst', 0),
-                                    n_bits=48,
-                                    src=('eth_src', 0)),
-                                ofpp.NXFlowSpecLoad(
-                                    dst=('vlan_tci', 0),
-                                    n_bits=16,
-                                    src=0),
-                                ofpp.NXFlowSpecLoad(
-                                    dst=('tunnel_id', 0),
-                                    n_bits=64,
-                                    src=('tunnel_id', 0)),
-                                ofpp.NXFlowSpecOutput(
-                                    dst='',
-                                    n_bits=32,
-                                    src=('in_port', 0)),
-                            ],
-                            table_id=20),
-                        ofpp.OFPActionOutput(patch_int_ofport, 0),
-                    ]),
-                ],
-                match=ofpp.OFPMatch(),
-                priority=1,
-                table_id=10),
-                           active_bundle=None),
+                           active_bundle=None)]
+        expected += self._get_learn_flows(ofpp, patch_int_ofport)
+        expected += [
             call._send_msg(ofpp.OFPFlowMod(dp,
                 cookie=self.stamp,
                 instructions=[ofpp.OFPInstructionGotoTable(table_id=22)],
@@ -280,6 +280,33 @@ class OVSTunnelBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase,
         ]
         self.assertEqual(expected, self.mock.mock_calls)
 
+    def _test_setup_default_table_dvr_helper(self, dvr_enabled):
+        patch_int_ofport = 5555
+        arp_responder_enabled = True
+        self.br.setup_default_table(patch_int_ofport=patch_int_ofport,
+            arp_responder_enabled=arp_responder_enabled,
+            dvr_enabled=dvr_enabled)
+        (dp, ofp, ofpp) = self._get_dp()
+        non_dvr_specific_call = call._send_msg(
+            ofpp.OFPFlowMod(
+                dp,
+                cookie=self.stamp,
+                instructions=[ofpp.OFPInstructionGotoTable(table_id=2)],
+                match=ofpp.OFPMatch(in_port=patch_int_ofport),
+                priority=1, table_id=0),
+            active_bundle=None)
+
+        if dvr_enabled:
+            self.assertNotIn(non_dvr_specific_call, self.mock.mock_calls)
+        else:
+            self.assertIn(non_dvr_specific_call, self.mock.mock_calls)
+
+    def test_setup_default_table_dvr_enabled(self):
+        self._test_setup_default_table_dvr_helper(dvr_enabled=True)
+
+    def test_setup_default_table_dvr_disabled(self):
+        self._test_setup_default_table_dvr_helper(dvr_enabled=False)
+
     def test_provision_local_vlan(self):
         network_type = 'vxlan'
         lvid = 888
@@ -405,45 +432,6 @@ class OVSTunnelBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase,
         ]
         self.assertEqual(expected, self.mock.mock_calls)
 
-    def test_install_arp_responder(self):
-        vlan = 3333
-        ip = '192.0.2.1'
-        mac = '08:60:6e:7f:74:e7'
-        self.br.install_arp_responder(vlan=vlan, ip=ip, mac=mac)
-        (dp, ofp, ofpp) = self._get_dp()
-        expected = [
-            call._send_msg(ofpp.OFPFlowMod(dp,
-                cookie=self.stamp,
-                instructions=[
-                    ofpp.OFPInstructionActions(ofp.OFPIT_APPLY_ACTIONS, [
-                        ofpp.OFPActionSetField(arp_op=self.arp.ARP_REPLY),
-                        ofpp.NXActionRegMove(
-                            dst_field='arp_tha',
-                            n_bits=48,
-                            src_field='arp_sha'),
-                        ofpp.NXActionRegMove(
-                            dst_field='arp_tpa',
-                            n_bits=32,
-                            src_field='arp_spa'),
-                        ofpp.OFPActionSetField(arp_sha=mac),
-                        ofpp.OFPActionSetField(arp_spa=ip),
-                        ofpp.NXActionRegMove(src_field='eth_src',
-                                             dst_field='eth_dst',
-                                             n_bits=48),
-                        ofpp.OFPActionSetField(eth_src=mac),
-                        ofpp.OFPActionOutput(ofp.OFPP_IN_PORT, 0),
-                    ]),
-                ],
-                match=ofpp.OFPMatch(
-                    eth_type=self.ether_types.ETH_TYPE_ARP,
-                    arp_tpa=ip,
-                    vlan_vid=vlan | ofp.OFPVID_PRESENT),
-                priority=1,
-                table_id=21),
-                           active_bundle=None),
-        ]
-        self.assertEqual(expected, self.mock.mock_calls)
-
     def test_delete_arp_responder(self):
         vlan = 3333
         ip = '192.0.2.1'
