@@ -27,6 +27,7 @@ from oslo_config import cfg
 from oslo_utils import uuidutils
 import testtools
 
+from neutron.common import config
 from neutron.objects import ports as port_obj
 from neutron.plugins.ml2.extensions import dns_integration
 from neutron.services.externaldns.drivers.designate import driver
@@ -53,6 +54,7 @@ class DNSIntegrationTestCase(test_plugin.Ml2PluginV2TestCase):
     _domain = DNSDOMAIN
 
     def setUp(self):
+        config.register_common_config_options()
         cfg.CONF.set_override('extension_drivers',
                               self._extension_drivers,
                               group='ml2')
@@ -80,7 +82,7 @@ class DNSIntegrationTestCase(test_plugin.Ml2PluginV2TestCase):
             net_kwargs['arg_list'] = \
                 net_kwargs.get('arg_list', ()) + (dns_apidef.DNSDOMAIN,)
         res = self._create_network(self.fmt, 'test_network', True,
-                                   **net_kwargs)
+                                   as_admin=True, **net_kwargs)
         network = self.deserialize(self.fmt, res)
         if ipv4:
             cidr = '10.0.0.0/24'
@@ -124,10 +126,10 @@ class DNSIntegrationTestCase(test_plugin.Ml2PluginV2TestCase):
     def _update_port_for_test(self, port, new_dns_name=NEWDNSNAME,
                               new_dns_domain=None, **kwargs):
         mock_client.reset_mock()
-        ip_addresses = [netaddr.IPAddress(ip['ip_address'])
-                        for ip in port['fixed_ips']]
-        records_v4 = [ip for ip in ip_addresses if ip.version == 4]
-        records_v6 = [ip for ip in ip_addresses if ip.version == 6]
+        records_v4 = [ip['ip_address'] for ip in port['fixed_ips']
+                      if netaddr.IPAddress(ip['ip_address']).version == 4]
+        records_v6 = [ip['ip_address'] for ip in port['fixed_ips']
+                      if netaddr.IPAddress(ip['ip_address']).version == 6]
         recordsets = []
         if records_v4:
             recordsets.append({'id': V4UUID, 'records': records_v4})
@@ -758,83 +760,6 @@ class DNSDomainPortsTestCase(DNSIntegrationTestCase):
             dns_data_db, dns_data_db_1, dns_data_db_2)
 
 
-class TestDesignateClientKeystoneV2(testtools.TestCase):
-    """Test case for designate clients """
-
-    TEST_URL = 'http://127.0.0.1:9001/v2'
-    TEST_ADMIN_USERNAME = uuidutils.generate_uuid(dashed=False)
-    TEST_ADMIN_PASSWORD = uuidutils.generate_uuid(dashed=False)
-    TEST_ADMIN_TENANT_NAME = uuidutils.generate_uuid(dashed=False)
-    TEST_ADMIN_TENANT_ID = uuidutils.generate_uuid(dashed=False)
-    TEST_ADMIN_AUTH_URL = 'http://127.0.0.1:35357/v2.0'
-    TEST_CA_CERT = uuidutils.generate_uuid(dashed=False)
-
-    TEST_CONTEXT = mock.Mock()
-    TEST_CONTEXT.auth_token = uuidutils.generate_uuid(dashed=False)
-
-    def setUp(self):
-        super(TestDesignateClientKeystoneV2, self).setUp()
-        cfg.CONF.set_override('url',
-                              self.TEST_URL,
-                              group='designate')
-        cfg.CONF.set_override('admin_username',
-                              self.TEST_ADMIN_USERNAME,
-                              group='designate')
-        cfg.CONF.set_override('admin_password',
-                              self.TEST_ADMIN_PASSWORD,
-                              group='designate')
-        cfg.CONF.set_override('admin_auth_url',
-                              self.TEST_ADMIN_AUTH_URL,
-                              group='designate')
-        cfg.CONF.set_override('admin_tenant_id',
-                              self.TEST_ADMIN_TENANT_ID,
-                              group='designate')
-        cfg.CONF.set_override('admin_tenant_name',
-                              self.TEST_ADMIN_TENANT_NAME,
-                              group='designate')
-
-        # enforce session recalculation
-        mock.patch.object(driver, '_SESSION', new=None).start()
-        self.driver_session = (
-            mock.patch.object(session, 'Session').start())
-        self.load_auth = (
-            mock.patch.object(driver.loading,
-                'load_auth_from_conf_options').start())
-        self.password = (
-            mock.patch.object(driver.password, 'Password').start())
-
-    def test_insecure_client(self):
-        cfg.CONF.set_override('insecure',
-                              True,
-                              group='designate')
-        driver.get_clients(self.TEST_CONTEXT)
-        args, kwargs = self.driver_session.call_args
-        self.assertIn('verify', kwargs)
-        self.assertFalse(kwargs['verify'], False)
-
-    def test_secure_client(self):
-        cfg.CONF.set_override('insecure',
-                              False,
-                              group='designate')
-        cfg.CONF.set_override('cafile',
-                              self.TEST_CA_CERT,
-                              group='designate')
-        driver.get_clients(self.TEST_CONTEXT)
-        args, kwargs = self.driver_session.call_args
-        self.assertIn('verify', kwargs)
-        self.assertEqual(kwargs['verify'], self.TEST_CA_CERT)
-
-    def test_auth_type_not_defined(self):
-        driver.get_clients(self.TEST_CONTEXT)
-        self.load_auth.assert_not_called()
-        self.password.assert_called_with(
-            auth_url=self.TEST_ADMIN_AUTH_URL,
-            password=self.TEST_ADMIN_PASSWORD,
-            tenant_id=self.TEST_ADMIN_TENANT_ID,
-            tenant_name=self.TEST_ADMIN_TENANT_NAME,
-            username=self.TEST_ADMIN_USERNAME)
-
-
 class TestDesignateClientKeystoneV3(testtools.TestCase):
     """Test case for designate clients """
 
@@ -888,8 +813,6 @@ class TestDesignateClientKeystoneV3(testtools.TestCase):
         self.load_auth = (
             mock.patch.object(driver.loading,
                 'load_auth_from_conf_options').start())
-        self.password = (
-            mock.patch.object(driver.password, 'Password').start())
 
     def test_insecure_client(self):
         cfg.CONF.set_override('insecure',
@@ -915,4 +838,3 @@ class TestDesignateClientKeystoneV3(testtools.TestCase):
     def test_auth_type_password(self):
         driver.get_clients(self.TEST_CONTEXT)
         self.load_auth.assert_called_with(cfg.CONF, 'designate')
-        self.password.assert_not_called()
